@@ -23,7 +23,6 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger(__name__)
 
-
 RECORD_MANAGER_FILENAME = "record_manager_cache.sqlite" # embed dataset recipe
 DSS_RECORD_MANAGER_FILENAME = "dss_record_manager_cache.sqlite" # embed document recipe
 KB_STATUS_FILENAME = "kb_status.json"
@@ -185,7 +184,10 @@ class DkuVectorStore:
     def add_filter(self, search_kwargs: Dict, filter_ : Dict) -> None:
         if not self.document_filter:
             raise Exception("Explicit filter not implemented on this vector store")
-        self.document_filter.add_filter(search_kwargs, filter_)
+        metadata_columns = [c.get("name") for c in self.kb.get("metadataColumnsSchema", [])]
+        cleaned_filter = DkuVectorStore._clean_clause(filter_, metadata_columns)
+        if cleaned_filter is not None:
+            self.document_filter.add_filter(search_kwargs, cleaned_filter)
 
     @staticmethod
     def _clean_filter(filter_: Dict, metadata_columns: List[str], vector_store_query_tool_ref: Optional[str]) -> Optional[Dict]:
@@ -257,6 +259,21 @@ class DkuVectorStore:
         else:
             raise ValueError("Unknown knowledge bank update method %s" % update_method)
 
+    def load_documents_to_add(self, documents_loader:VectorStoreLoader, embeddings: DKUEmbeddings, to_add_documents_uuids: Optional[list[str]]=None)-> "VectorStore":
+        if len(to_add_documents_uuids) > 0:
+            logger.info("Performing embedding : adding {} total new langchain documents in the vector store".format(len(to_add_documents_uuids)))
+            vector_store_db = self.add_documents(documents_loader, embeddings, to_add_documents_uuids)
+            logger.info("Finished adding langchain documents")
+            return vector_store_db
+        else:
+            logger.info("No langchain document to add in the vector store")
+            return None
+
+    def load_documents_to_delete(self, embeddings: DKUEmbeddings, to_delete_documents_uuids:  Optional[list[str]]=None):
+        logger.info("Deleting {} outdated langchain documents in the vector store".format(len(to_delete_documents_uuids)))
+        vector_store_db = self.delete_documents(to_delete_documents_uuids, embeddings)
+        return vector_store_db
+
     def _overwrite_documents(self, documents_loader: VectorStoreLoader, embeddings: DKUEmbeddings) -> "VectorStore":
         logger.info("Adding documents to the vector store, in overwrite mode")
         if self.source_id_column is not None:
@@ -286,12 +303,14 @@ class DkuVectorStore:
         self._index_documents(documents_loader, vectorstore_db, cleanup_mode=RecordManagerCleanupMode.INCREMENTAL, source_id_key=self.source_id_column)
         return vectorstore_db
 
-    def delete_documents(self, documents_uuids:  list[str], embeddings: DKUEmbeddings):
+    def delete_documents(self, documents_uuids:  list[str], embeddings: DKUEmbeddings) -> "VectorStore":
         if len(documents_uuids) > 0:
             vectorstore_db = self.get_db(embeddings=embeddings, allow_creation=False)
             vectorstore_db.delete(ids=documents_uuids)
+            return vectorstore_db
+        return None
 
-    def add_documents(self, documents_loader: VectorStoreLoader, embeddings: DKUEmbeddings, documents_uuids:  Optional[list[str]]=None) -> None:
+    def add_documents(self, documents_loader: VectorStoreLoader, embeddings: DKUEmbeddings, documents_uuids:  Optional[list[str]]=None) -> "VectorStore":
         vectorstore_db = self.get_db(embeddings, allow_creation=True)
         total = 0
         nb_batches = 0
@@ -313,7 +332,10 @@ class DkuVectorStore:
         return vectorstore_db
 
     def _index_documents(self, documents_loader: VectorStoreLoader, vectorstore_db: "VectorStore", cleanup_mode: RecordManagerCleanupMode, source_id_key: str, batch_size: int=100) -> None:  # 100 is the default batch size used by RecordManager
-        from langchain.indexes import index
+        try:
+            from langchain_classic.indexes import index
+        except ModuleNotFoundError:
+            from langchain.indexes import index
 
         batch_size = self._check_batch_size_against_sqlite_version(batch_size)
         record_manager = self._get_record_manager()
@@ -325,6 +347,20 @@ class DkuVectorStore:
                 ret = self.transform_document_before_load(doc)
                 yield ret
 
+        def need_patching_for_batched_delete() -> bool:
+            """
+            Patch is no longer needed after sc-241245
+            langchain PR: https://github.com/langchain-ai/langchain/pull/31206 released on langchain-core 0.3.60
+            """
+            try:
+                from packaging.version import parse
+                import langchain_core
+                return parse(langchain_core.__version__) < parse("0.3.60")
+            except Exception:
+                # Returning true to ensure the behavior isn't changed if we have any issue
+                logger.warning("Could not determine langchain-core version, applying batched delete patch by default.")
+                return True
+
         def patch_for_batched_delete() -> None:
             """
             Warning: this method breaks the query into multiple transactions,
@@ -332,6 +368,7 @@ class DkuVectorStore:
 
             That is already happening for RecordManagerCleanupMode.FULL anyway.
             """
+            logger.info("We need to patch RecordManager.delete_keys to use batched deletes to avoid sqlite limitations.")
             non_batched_delete = record_manager.delete_keys
             def batched_delete_keys(self, keys: Sequence[str]) -> None:
                 # batch the keys based on batch_size
@@ -341,7 +378,7 @@ class DkuVectorStore:
             from types import MethodType
             record_manager.delete_keys = MethodType(batched_delete_keys, record_manager)
 
-        if cleanup_mode == RecordManagerCleanupMode.INCREMENTAL:
+        if cleanup_mode == RecordManagerCleanupMode.INCREMENTAL and need_patching_for_batched_delete():
             patch_for_batched_delete()
 
         index_info = index(
@@ -424,7 +461,10 @@ class DkuVectorStore:
         }
 
     def _get_record_manager(self):
-        from langchain.indexes import SQLRecordManager
+        try:
+            from langchain_classic.indexes import SQLRecordManager
+        except ModuleNotFoundError:
+            from langchain.indexes import SQLRecordManager
 
         record_manager_file_path = os.path.join(self.exec_folder, RECORD_MANAGER_FILENAME)
         record_manager_already_exists = os.path.exists(record_manager_file_path)
