@@ -234,10 +234,12 @@ app.controller("PluginSummaryController", function ($scope, $filter) {
 });
 
 
-app.controller("PluginSettingsController", function ($scope, PluginConfigUtils, DataikuAPI, WT1, $stateParams, CreateModalFromTemplate, Dialogs) {
+app.controller("PluginSettingsController", function ($scope, PluginConfigUtils, DataikuAPI, WT1, $stateParams, CreateModalFromTemplate, Dialogs, PermissionsDataFetcher) {
 
     $scope.pluginsUIState = $scope.pluginsUIState || {};
     $scope.pluginsUIState.settingsPane = $stateParams.selectedTab || 'parameters';
+    
+    $scope.permissionsDataFetcher = PermissionsDataFetcher($scope);
 
     $scope.hooks = {};
 
@@ -1142,12 +1144,13 @@ app.controller("PluginsExploreController", function ($rootScope, $scope, $contro
 
             const supportLevelMap = {SUPPORTED: 2, TIER2_SUPPORT: 1, NOT_SUPPORTED: 0, NONE: -1};
             function rank(plugin) {
+                const untagged = plugin.storeDesc?.meta?.tags.length > 0 ? 1 : 0;
                 const updateAvailable = plugin.updateAvailable ? 1 : 0;
                 const notInstalled = plugin.installed ? 0 : 1;
                 const desc = plugin.storeDesc||{};
                 const meta = desc.meta||{};
                 const supportLevel = supportLevelMap[meta.supportLevel||'NONE'];
-                return [updateAvailable, notInstalled, supportLevel];
+                return [untagged, updateAvailable, notInstalled, supportLevel];
             }
             $scope.pluginsList.plugins.sort((a,b) => rank(a) > rank(b) ? -1 : 1);
         }).error(setErrorInScope.bind($scope));
@@ -1693,9 +1696,10 @@ app.directive('pluginPresetSecurityPermissions', function(PermissionsService) {
         restrict : 'A',
         templateUrl : '/templates/plugins/plugin-preset-security-permissions.html',
         scope : {
-            preset  : '='
+            preset  : '=',
+            permissionsDataFetcher: '<'
         },
-        link : function($scope, element, attrs) {
+        link : function($scope) {
             $scope.ui = {};
 
             $scope.securityPermissionsHooks = {};
@@ -1731,7 +1735,8 @@ app.directive('pluginParameterSetSecurityPermissions', function() {
         restrict : 'A',
         templateUrl : '/templates/admin/plugins/plugin-parameter-set-security-permissions.html',
         scope : {
-            parameterSet  : '='
+            parameterSet  : '=',
+            permissionsDataFetcher: '<'
         },
         link : function($scope, element, attrs) {
             $scope.securityPermissionsHooks = {};
@@ -1758,14 +1763,15 @@ app.directive('pluginParameterSetSecurityPermissions', function() {
 });
 
 
-app.directive('pluginSecurityPermissions', function(DataikuAPI, PermissionsService) {
+app.directive('pluginSecurityPermissions', function() {
     return {
         restrict : 'A',
         templateUrl : '/templates/plugins/plugin-security-permissions.html',
         scope : {
-            pluginSettings : '='
+            pluginSettings : '=',
+            permissionsDataFetcher: '<'
         },
-        link : function($scope, element, attrs) {
+        link : function($scope) {
             $scope.securityPermissionsHooks = {};
             $scope.securityPermissionsHooks.makeNewPerm = function() {
                 return {
@@ -1785,7 +1791,7 @@ app.directive('pluginSecurityPermissions', function(DataikuAPI, PermissionsServi
 });
 
 
-app.directive('securityPermissionsBase', function(DataikuAPI, $rootScope, PermissionsService) {
+app.directive('securityPermissionsBase', function($rootScope, PermissionsService) {
     return {
         restrict : 'A',
         scope : false,
@@ -1806,6 +1812,7 @@ app.directive('securityPermissionsBase', function(DataikuAPI, $rootScope, Permis
                 /* Handle implied permissions */
                 $scope.securityPermissionsHooks.fixupPermissionItem($scope[attrs.permissionsBearer].defaultPermission);
             };
+            
             const fixupPermissions = function() {
                 if (!$scope[attrs.permissionsBearer] || !$scope.securityPermissionsHooks) return;
                 /* Handle implied permissions */
@@ -1815,14 +1822,15 @@ app.directive('securityPermissionsBase', function(DataikuAPI, $rootScope, Permis
                 });
             };
 
-            DataikuAPI.security.listGroups(false).success(function(allGroups) {
-                $scope.allGroups = allGroups;
-                DataikuAPI.security.listUsers().success(function(data) {
-                    $scope.allUsers = data.sort((a, b) => a.displayName.localeCompare(b.displayName));
-                    $scope.allUsersLogin = data.map(user => '@' + user.login);
-                }).error(setErrorInScope.bind($scope));
-                $scope.unassignedGroups = PermissionsService.buildUnassignedGroups($scope[attrs.permissionsBearer], $scope.allGroups);
-            }).error(setErrorInScope.bind($scope));
+            $scope.permissionsDataFetcher.getUsersAndGroups().then(function(result) {
+               $scope.allUsers = result.allUsers;
+               $scope.allGroups = result.allGroups;
+               $scope.allUsersLogin = result.allUsersLogin;
+                if ($scope[attrs.permissionsBearer]) {
+                    $scope.unassignedGroups = PermissionsService.buildUnassignedGroups($scope[attrs.permissionsBearer], $scope.allGroups);
+                }
+                fixupPermissions();
+            });
 
             $scope.addPermission = function() {
                 $scope[attrs.permissionsBearer].permissions.push($scope.newPerm);
@@ -1849,8 +1857,6 @@ app.directive('securityPermissionsBase', function(DataikuAPI, $rootScope, Permis
                 fixupDefaultPermission();
                 fixupPermissions();
             }, false)
-            $scope.unassignedGroups = PermissionsService.buildUnassignedGroups($scope[attrs.permissionsBearer], $scope.allGroups);
-            fixupPermissions();
         }
     };
 });
@@ -1894,7 +1900,8 @@ app.directive('pluginParameterSet', function(CreateModalFromTemplate) {
             pluginDesc : '=',
             presets : '=',
             remove : '=',
-            allPresets : '='
+            allPresets : '=',
+            permissionsDataFetcher: '<'
         },
         link : function($scope, element, attrs) {
             $scope.$watch('allPresets', function() {
@@ -1922,7 +1929,7 @@ app.directive('pluginParameterSet', function(CreateModalFromTemplate) {
 });
 
 
-app.directive('pluginPreset', function(DataikuAPI) {
+app.directive('pluginPreset', function() {
     return {
         restrict : 'A',
         templateUrl : '/templates/plugins/plugin-preset.html',
@@ -1930,12 +1937,14 @@ app.directive('pluginPreset', function(DataikuAPI) {
             preset : '=',
             parameterSetDesc : '=',
             pluginDesc : '=',
-            remove : '='
+            remove : '=',
+            permissionsDataFetcher: '<'
         },
         link : function($scope) {
-            DataikuAPI.plugins.canAdminPlugin($scope.pluginDesc.id).success(function(data) {
-                $scope.canAdminPlugin = data;
-            }).error(setErrorInScope.bind($scope));
+            $scope.canAdminPlugin = undefined;
+            $scope.permissionsDataFetcher.canAdminPlugin($scope.pluginDesc.id).then(function(canAdmin) {
+                $scope.canAdminPlugin = canAdmin;
+            });
 
         	$scope.$watch('preset', function() {
         		if (!$scope.preset) return;
@@ -1980,7 +1989,7 @@ app.directive('pluginAppTemplate', function(DataikuAPI) {
             $scope.availableConnections = [];
             DataikuAPI.admin.connections.list().success(function(data) {
                 angular.forEach(data, function(c, n) {
-                    $scope.availableConnections.push({name:n, type:c.type});
+                    $scope.availableConnections.push({name:n, type:c.type, description: c.description});
                 });
             }).error(setErrorInScope.bind($scope));
             
