@@ -1203,7 +1203,7 @@ app.component("tooManyDroppedRowsWarning", {
     }
 });
 
-    app.service("TimeseriesFeatureGenerationService", function() {
+app.service("TimeseriesFeatureGenerationService", function($timeout) {
         this.cleanUpSettings = function(mlTaskDesign) {
             /*
             Cleans the timeseries shift values of an ML Task for time-aware feature engineering.
@@ -1215,55 +1215,286 @@ app.component("tooManyDroppedRowsWarning", {
 
             for (var featureName in mlTaskDesign.preprocessing.feature_generation.shifts) {
                 const toCheckFromForecast = mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_forecast;
-                if (typeof toCheckFromForecast === "string") {
-                    mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_forecast = [...new Set(Array.from(toCheckFromForecast.split(",")).map(x => parseInt(x)).filter(x => !isNaN(x)))];
-                }
+                mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_forecast = this.parseArrayFromStringIfNeeded(toCheckFromForecast);
+
                 const toCheckFromHorizon = mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_horizon;
-                if (typeof toCheckFromHorizon === "string") {
-                    mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_horizon = [...new Set(Array.from(toCheckFromHorizon.split(",")).map(x => parseInt(x)).filter(x => !isNaN(x)))];
-                }
+                mlTaskDesign.preprocessing.feature_generation.shifts[featureName].from_horizon = this.parseArrayFromStringIfNeeded(toCheckFromHorizon);
             }
         };
+
+        this.parseArrayFromStringIfNeeded = function (input) {
+            if (typeof input === "string") {
+                return [...new Set(Array.from(input.split(",")).map(x => parseInt(x)).filter(x => !isNaN(x)))];
+            }
+            return input;
+        }
+
+        this.isMassEditMode = function(scope, selection) {
+            let newState =  selection.selectedObjects && selection.selectedObjects.length > 0;
+
+            // when going from the mass edit to the per-feature mode, the table can break
+            // to prevent this, we force a rerender
+            if(scope._isMassEditMode && !newState) {
+                if (scope.shouldRerender) {
+                    $timeout(function () {
+                        scope.$broadcast("redrawFatTable");
+                    }, 0);
+                    scope.shouldRerender = false;
+                }
+            }
+
+            scope._isMassEditMode = newState;
+            return newState;
+        };
+
+        this.getTableSize = function (numberOfElements, rowHeight) {
+           return Math.min(numberOfElements, 10) * rowHeight + 10;
+        }
     });
 
-    app.controller('FeatureGenerationController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService, TimeseriesForecastingUtils) {
+app.controller('FeatureGenerationController', function($scope, CreateModalFromTemplate) {
+    $scope.openWindowsSettingsModal = function() {
+        CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-settings-modal.html", $scope, null, function(newScope) {
+            newScope.windows_rescale_numericals = $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals;
+            newScope.windows_max_categories = $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories;
+
+            newScope.save = () => {
+                $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals = newScope.windows_rescale_numericals;
+                $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories = newScope.windows_max_categories;
+                newScope.dismiss();
+            }
+        }, false, false, true)
+    };
+})
+
+app.controller('FeatureGenerationShiftController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService) {
+        // ================= Mass operations
+        $scope._isMassEditMode = false; // used by TimeseriesFeatureGenerationService
+
+        $scope.massEditInputIsDirty = false; // whether or not the user interacted with the mass edit inputs
+
+        $scope.$watch('selection', function() {
+            // erase the dirty state when the user deselects everything
+            if ($scope.selection.selectedObjects && $scope.selection.selectedObjects.length === 0) {
+                $scope.massEditInputIsDirty = false;
+            }
+            $scope.initMassEditValues();
+        }, true);
+
+        $scope.setMassEditDirty = function () {
+            $scope.massEditInputIsDirty = true;
+        };
+
+        let redrawFn = function() {
+            $scope.shouldRerender = true;
+        };
+
+        $scope.setMassEditTargetShiftMode = function (target) {
+            $scope.massEditTargetShiftMode = target;
+        };
+
+        $scope.applyMassEditShifts = function () {
+            $scope.setSelectedFeaturesShifts();
+            if ($scope.massEditTargetShiftMode === "AUTO") {
+                $scope.setSelectedFeaturesToAutoShift();
+            }
+            if ($scope.massEditTargetShiftMode === "FIXED") {
+                $scope.setSelectedFeaturesToFixedShifts();
+            }
+
+            $scope.massEditInputIsDirty = false;
+            $scope.initMassEditValues();
+        };
+
+        $(window).on('resize', redrawFn);
+
+        $scope.$on('$destroy', function() {
+            $(window).off('resize', redrawFn);
+        });
+
+        $scope.isMassEditMode = function(selection) {
+            return TimeseriesFeatureGenerationService.isMassEditMode($scope, selection);
+        };
+
+        $scope.getTableSize = function(numberOfElements, rowHeight) {
+            return TimeseriesFeatureGenerationService.getTableSize(numberOfElements, rowHeight);
+        };
+
+        $scope.initMassEditValues = function () {
+            // If all features have identical forecast or horizon shifts, this function sets the value of the mass
+            // selection shifts to the common value. Otherwise, it defaults to empty shifts.
+            if (!$scope.selection.selectedObjects || $scope.selection.selectedObjects.length === 0) {
+                return;
+            }
+
+            if (!$scope.massEditInputIsDirty) {
+                $scope.initMassEditShifts();
+                $scope.initMassEditTargetShiftMode();
+            }
+
+            $scope.validateMassEditShifts();
+        };
+
+        $scope.initMassEditShifts = function () {
+            let forecastShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_forecast;
+
+            forecastShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(forecastShifts)
+
+            let forecastShiftsAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    let featureShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast;
+                    featureShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(featureShifts);
+                    return featureShifts.length === forecastShifts.length && forecastShifts.every(v => featureShifts.includes(v));
+                }
+            )
+            if (forecastShiftsAreIdentical) {
+                $scope.uiState.massActionForecastShifts = forecastShifts;
+            }
+            else {
+                $scope.uiState.massActionForecastShifts = [];
+            }
+
+            let horizonShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_horizon;
+
+            horizonShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(horizonShifts);
+
+            let horizonShiftsAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    let featureShifts = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon;
+                    featureShifts = TimeseriesFeatureGenerationService.parseArrayFromStringIfNeeded(featureShifts);
+                    return featureShifts.length === horizonShifts.length && horizonShifts.every(v => featureShifts.includes(v));
+                }
+            )
+
+            if (horizonShiftsAreIdentical) {
+                $scope.uiState.massActionHorizonShifts = horizonShifts;
+            }
+            else {
+                $scope.uiState.massActionHorizonShifts = [];
+            }
+        }
+
+        $scope.initMassEditTargetShiftMode = function () {
+            let autoShiftMode = $scope.mlTaskDesign.preprocessing.feature_generation.shifts[$scope.selection.selectedObjects[0]._name].from_horizon_mode;
+
+            let autoShiftModesAreIdentical = $scope.selection.selectedObjects.every(
+                (feature) => {
+                    return autoShiftMode === $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode;
+                }
+            )
+            $scope.massEditTargetShiftMode = autoShiftModesAreIdentical ? autoShiftMode : null;
+        }
+
+        $scope.getFeatureHavingShiftModeNumber = function (autoShiftMode) {
+            if (!$scope.massEditTargetShiftMode) {
+                return $scope.someSelectedFeaturesAreInMode(autoShiftMode);
+            }
+            if (autoShiftMode !== $scope.massEditTargetShiftMode) {
+                if (autoShiftMode === "FIXED") {
+                    return $scope.someSelectedFeaturesDontSupportAutoShift();
+                }
+            }
+            return false;
+        };
+
+        $scope.autoShiftModeShouldBeStriped = function (autoShiftMode) {
+            if (!$scope.massEditTargetShiftMode) {
+                return $scope.someSelectedFeaturesAreInMode(autoShiftMode);
+            }
+            if (autoShiftMode !== $scope.massEditTargetShiftMode) {
+                if (autoShiftMode === "FIXED") {
+                    return $scope.someSelectedFeaturesDontSupportAutoShift();
+                }
+            }
+            return false;
+        };
 
         $scope.selectedFeaturesContainsPastCovariate = function () {
             return $scope.selection.selectedObjects.some(feature => $scope.isPastOnlyFeature(feature._name));
         };
-        $scope.allSelectedFeaturesSupportAutoShift = function () {
+
+        $scope.someSelectedFeaturesSupportAutoShift = function () {
+            return $scope.selection.selectedObjects.some(feature => $scope.featureSupportsAutoShifts(feature));
+        };
+
+        $scope.someSelectedFeaturesDontSupportAutoShift = function () {
             return $scope.selection.selectedObjects.some(feature => !$scope.featureSupportsAutoShifts(feature));
         };
 
-        $scope.setFeatureShifts = function(horizonShifts, forecastShifts, enableHorizonShiftsAuto) {
+        $scope.someSelectedFeaturesArePastOnly = function () {
+            return $scope.selection.selectedObjects.some(feature => $scope.isPastOnlyFeature(feature._name));
+        };
+
+        $scope.setSelectedFeaturesToAutoShift = function (){
+            $scope.selection.selectedObjects.forEach(
+                (feature) => {
+                    if ($scope.featureSupportsAutoShifts(feature)) {
+                        $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'AUTO';
+                    }
+                }
+            )
+            if (document.activeElement) {
+                document.activeElement.blur();
+            }
+        };
+
+        $scope.allSelectedFeaturesAreInMode = function (mode) {
+            return $scope.selection.selectedObjects.every(
+                (feature) => {
+                    return $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode === mode;
+                }
+            );
+        }
+
+        $scope.someSelectedFeaturesAreInMode = function (mode) {
+            return $scope.selection.selectedObjects.some(
+                (feature) => {
+                    return $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode === mode;
+                }
+            );
+        }
+
+        $scope.setSelectedFeaturesToFixedShifts = function (){
+            $scope.selection.selectedObjects.forEach(
+                (feature) => {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'FIXED';
+                }
+            )
+        };
+
+        $scope.getFeatureSupportingAutoShiftNumber = function () {
+            return $scope.selection.selectedObjects.filter(
+                (feature) => $scope.featureSupportsAutoShifts(feature)
+            ).length;
+        };
+
+        $scope.setSelectedFeaturesShifts = function() {
             $scope.selection.selectedObjects.forEach(function(feature) {
                 if (!$scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name]) {
                     $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name] = {};
                 }
-                $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast = forecastShifts || "";
-                if (enableHorizonShiftsAuto) {
-                    if ($scope.featureSupportsAutoShifts(feature)) {
-                        $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'AUTO';
-                    } else {
-                        // Do not update horizon shifts for feature not supporting auto
-                    }
-                } else {
-                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon = horizonShifts || "";
-                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon_mode = 'FIXED';
+                $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_forecast = $scope.uiState.massActionForecastShifts || "";
+                if ($scope.massEditTargetShiftMode !== "AUTO") {
+                    $scope.mlTaskDesign.preprocessing.feature_generation.shifts[feature._name].from_horizon = $scope.uiState.massActionHorizonShifts || "";
                 }
-
             });
             $scope.validateShifts();
+            $scope.validateMassEditShifts();
         };
 
-        $scope.openShiftsModal = function() {
-            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-edit-feature-shift-modal.html", $scope, null, function(newScope) {
-                newScope.save = () => {
-                    $scope.setFeatureShifts(newScope.horizonShifts, newScope.forecastShifts, newScope.enableHorizonShiftsAuto);
-                    newScope.dismiss();
-                }
-            }, false, false, true)
-        };
+        $scope.validateMassEditShifts = function () {
+            $scope.uiState.massEditShiftsValidationErrors = {}
+
+            let shiftFromForecastAsString = ($scope.uiState.massActionForecastShifts || "").toString();
+            let shiftFromHorizonAsString = ($scope.uiState.massActionHorizonShifts || "").toString();
+            let hasPastOnlyFeature = $scope.someSelectedFeaturesArePastOnly();
+
+            $scope.uiState.massEditShiftsValidationErrors = $scope.validateShiftsAsString(shiftFromForecastAsString, shiftFromHorizonAsString, hasPastOnlyFeature);
+
+            $scope.updateSelectedAlgosMissingRequiredExternalFeature();
+        }
+
+        // =================
 
         $scope.setFeatureAutoShiftsParams = function (maxSelectedHorizonShifts, minHorizonShiftPastOnly, maxHorizonShiftPastOnly, minHorizonShiftKnownInAdvance, maxHorizonShiftKnownInAdvance) {
             if (!$scope.mlTaskDesign.preprocessing.feature_generation.auto_shifts_params) {
@@ -1313,6 +1544,74 @@ app.component("tooManyDroppedRowsWarning", {
             }, false, false, true)
         };
 
+        $scope.$on('$destroy', function() {
+            // clean settings when leaving the tab
+            TimeseriesFeatureGenerationService.cleanUpSettings($scope.mlTaskDesign);
+         })
+    });
+
+app.controller('FeatureGenerationWindowController', function($scope, CreateModalFromTemplate, TimeseriesFeatureGenerationService, TimeseriesForecastingUtils) {
+
+        // ================= Mass operations
+        $scope._isMassEditMode = false; // used by TimeseriesFeatureGenerationService
+
+        $scope.initMassEditTargetOperations = function () {
+            $scope.massEditTargetOperations = {'numerical': [], 'categorical': []};
+        }
+
+        $scope.initMassEditTargetOperations();
+
+        let redrawFn = function() {
+            $scope.shouldRerender = true;
+        }
+
+        $(window).on('resize', redrawFn);
+
+        $scope.$on('$destroy', function() {
+            $(window).off('resize', redrawFn);
+        });
+
+        $scope.$watch('selection', function() {
+            if ($scope.selection.selectedObjects && $scope.selection.selectedObjects.length === 0) {
+                // if no more objects are selected, reset the mass edit state
+                $scope.initMassEditTargetOperations();
+            }
+            $scope.getMassEditOperations();
+        }, true);
+
+        $scope.isOperationTargetSet = function (operationName, operationType) {
+            // whether an operation is present in the massEditTargetOperations
+            return $scope.massEditTargetOperations[operationType].some(
+                operation => operation.operation === operationName
+            )
+        };
+
+        $scope.getOperationTarget = function (operationName, operationType) {
+            if ($scope.isOperationTargetSet(operationName, operationType)) {
+                return $scope.massEditTargetOperations[operationType].filter(
+                    operation => operation.operation === operationName
+                )[0];
+            }
+        };
+
+        $scope.isMassEditMode = function(selection) {
+            return TimeseriesFeatureGenerationService.isMassEditMode($scope, selection);
+        };
+
+        $scope.getTableSize = function(numberOfElements, rowHeight) {
+            return TimeseriesFeatureGenerationService.getTableSize(numberOfElements, rowHeight);
+        }
+
+        $scope.addToTargetOperations = function(operationName, enabled, operationType) {
+            if (!$scope.isOperationTargetSet(operationName, operationType)) {
+                $scope.massEditTargetOperations[operationType].push({operation: operationName, enabled: enabled})
+            } else {
+                $scope.getOperationTarget(operationName, operationType).enabled = enabled;
+            }
+
+        };
+        // =================
+
         $scope.setWindowOperations = function(numOperations, catOperations, windowIndex) {
             let selectedAndValidFeatures = $scope.uiState.windows[windowIndex].operations_list
                 .filter(el => {
@@ -1328,54 +1627,56 @@ app.component("tooManyDroppedRowsWarning", {
             $scope.validateWindows();
         }
 
+        $scope.getMassEditOperations = function () {
+            let per_feature = $scope.mlTaskDesign.preprocessing.per_feature;
+            let selectedFeaturesNumerical = $scope.window.operations_list.filter(x => per_feature[x[0]].type === "NUMERIC" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+            if (selectedFeaturesNumerical.length === 0) {
+                $scope.massEditTargetOperations['numerical'] = [];
+            }
+            let someDisabledOperationsNumerical = new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+            let allEnabledOperationsNumerical = (new Set(["MEAN", "MEDIAN", "STD", "MIN", "MAX"])).difference(someDisabledOperationsNumerical);
+            let partiallyEnabledOperationsNumerical = (new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsNumerical);
+            $scope.massEditOperationsNumerical = [
+                {operation: "MEAN", enabled: $scope.isOperationTargetSet("MEAN", 'numerical') ? $scope.getOperationTarget("MEAN", "numerical").enabled : allEnabledOperationsNumerical.has("MEAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEAN")},
+                {operation: "MEDIAN", enabled: $scope.isOperationTargetSet("MEDIAN", 'numerical') ? $scope.getOperationTarget("MEDIAN", "numerical").enabled : allEnabledOperationsNumerical.has("MEDIAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEDIAN")},
+                {operation: "STD", enabled: $scope.isOperationTargetSet("STD", 'numerical') ? $scope.getOperationTarget("STD", "numerical").enabled : allEnabledOperationsNumerical.has("STD"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("STD")},
+                {operation: "MIN", enabled: $scope.isOperationTargetSet("MIN", 'numerical') ? $scope.getOperationTarget("MIN", "numerical").enabled : allEnabledOperationsNumerical.has("MIN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MIN")},
+                {operation: "MAX", enabled: $scope.isOperationTargetSet("MAX", 'numerical') ? $scope.getOperationTarget("MAX", "numerical").enabled : allEnabledOperationsNumerical.has("MAX"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MAX")}
+            ];
 
-        $scope.openWindowsMassActionModal = function(windowIndex, window) {
-            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-mass-action-modal.html", $scope, null, function(newScope) {
-                let per_feature = $scope.mlTaskDesign.preprocessing.per_feature;
-                let selectedFeaturesNumerical = window.operations_list.filter(x => per_feature[x[0]].type === "NUMERIC" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
-                let someDisabledOperationsNumerical = new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
-                let allEnabledOperationsNumerical = (new Set(["MEAN", "MEDIAN", "STD", "MIN", "MAX"])).difference(someDisabledOperationsNumerical);
-                let partiallyEnabledOperationsNumerical = (new Set(selectedFeaturesNumerical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsNumerical);
-                newScope.operationsNumerical = [
-                    {operation: "MEAN", enabled: allEnabledOperationsNumerical.has("MEAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEAN")},
-                    {operation: "MEDIAN", enabled: allEnabledOperationsNumerical.has("MEDIAN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MEDIAN")},
-                    {operation: "STD", enabled: allEnabledOperationsNumerical.has("STD"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("STD")},
-                    {operation: "MIN", enabled: allEnabledOperationsNumerical.has("MIN"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MIN")},
-                    {operation: "MAX", enabled: allEnabledOperationsNumerical.has("MAX"), partiallyEnabled: partiallyEnabledOperationsNumerical.has("MAX")}
-                ];
-                let selectedFeaturesCategorical = window.operations_list.filter(x => per_feature[x[0]].type === "CATEGORY" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
-                let someDisabledOperationsCategorical = new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
-                let allEnabledOperationsCategorical = (new Set(["FREQUENCY"])).difference(someDisabledOperationsCategorical);
-                let partiallyEnabledOperationsCategorical = (new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsCategorical);
-                newScope.operationsCategorical = [
-                    {operation: "FREQUENCY", enabled: allEnabledOperationsCategorical.has("FREQUENCY"), partiallyEnabled: partiallyEnabledOperationsCategorical.has("FREQUENCY")},
-                ];
-                newScope.window = window;
-                newScope.atLeastOneNumericalSelected = function () {
-                    return selectedFeaturesNumerical.length > 0;
-                }
-                newScope.atLeastOneCategoricalSelected = function () {
-                    return selectedFeaturesCategorical.length > 0;
-                }
-                newScope.save = () => {
-                    $scope.setWindowOperations(newScope.operationsNumerical, newScope.operationsCategorical, windowIndex);
-                    newScope.dismiss();
-                }
-            }, false, false, true)
-        };
+            let selectedFeaturesCategorical = $scope.window.operations_list.filter(x => per_feature[x[0]].type === "CATEGORY" && x.$selected && TimeseriesForecastingUtils.isWindowCompatible(per_feature[x[0]]));
+            if (selectedFeaturesCategorical.length === 0) {
+                $scope.massEditTargetOperations['categorical'] = [];
+            }
+            let someDisabledOperationsCategorical = new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => !y.enabled).map(y => y.operation)));
+            let allEnabledOperationsCategorical = (new Set(["FREQUENCY"])).difference(someDisabledOperationsCategorical);
+            let partiallyEnabledOperationsCategorical = (new Set(selectedFeaturesCategorical.map(x => x[1]).flatMap(x => x.filter(y => y.enabled).map(y => y.operation)))).difference(allEnabledOperationsCategorical);
+            $scope.massEditOperationsCategorical = [
+                {operation: "FREQUENCY", enabled: $scope.isOperationTargetSet("FREQUENCY", 'categorical') ? $scope.getOperationTarget("FREQUENCY", "categorical").enabled : allEnabledOperationsCategorical.has("FREQUENCY"), partiallyEnabled: partiallyEnabledOperationsCategorical.has("FREQUENCY")},
+            ];
 
-        $scope.openWindowsSettingsModal = function() {
-            CreateModalFromTemplate("/templates/analysis/mlcommon/settings/time-series-windows-settings-modal.html", $scope, null, function(newScope) {
-                newScope.windows_rescale_numericals = $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals;
-                newScope.windows_max_categories = $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories;
+            $scope.numericalSelectedFeaturesNb = selectedFeaturesNumerical.length;
 
-                newScope.save = () => {
-                    $scope.mlTaskDesign.preprocessing.feature_generation.windows_rescale_numericals = newScope.windows_rescale_numericals;
-                    $scope.mlTaskDesign.preprocessing.feature_generation.windows_max_categories = newScope.windows_max_categories;
-                    newScope.dismiss();
-                }
-            }, false, false, true)
-        };
+            $scope.categoricalSelectedFeaturesNb = selectedFeaturesCategorical.length;
+        }
+
+        $scope.setMassEditOperations = function () {
+            for (let targetOperation of $scope.massEditTargetOperations['numerical'].concat($scope.massEditTargetOperations['categorical'])) {
+                let selectedFeature = $scope.selection.selectedObjects.map(feature => feature[0]);
+                $scope.window.operations_list.forEach(function (feature) {
+                    if (selectedFeature.includes(feature[0])) {
+                        for (let operation of feature[1]) {
+                            if (operation.operation === targetOperation.operation) {
+                                operation.enabled = targetOperation.enabled;
+                                break;
+                            }
+                        }
+                    }
+                })
+            }
+
+            $scope.getMassEditOperations();
+        }
 
         $scope.hasAnyEnabledOperation = function(window) {
             return window.operations_list.some(
@@ -1384,15 +1685,6 @@ app.component("tooManyDroppedRowsWarning", {
                 )
             );
         }
-
-        $scope.getTableSize = function(numberOfElements, rowHeight) {
-            return Math.min(numberOfElements, 10) * rowHeight + 10;
-        }
-
-        $scope.$on('$destroy', function() {
-            // clean settings when leaving the tab
-            TimeseriesFeatureGenerationService.cleanUpSettings($scope.mlTaskDesign);
-         })
     });
 
 app.component("cyclicalEncodingWarning", {
