@@ -25,6 +25,7 @@
             activeFilters: '<',
             filters: '<',
             dashboardTheme: '<',
+            canEditGrid: '<',
             loadingStateChange: '&', // ({ $isLoading }) => void
             tilesChange: '&', // ({ $tiles }) => void
             filtersChange: '&', // ({ $filters }) => void
@@ -33,10 +34,12 @@
             isDeactivatedChange: '&', // ({ $isDeactivated }) => void
             preselectedTileChange: '&', // ({ $preselectedTile }) => void
             gridInit: '&', // ({$api: { reload: () => void }}),
-            toggleGroupTileScrollBarDisplay: '&' // ({ $forceShow: boolean, $scrollTopOnHide: boolean }) => void
+            toggleGroupTileScrollBarDisplay: '&', // ({ $forceShow: boolean, $scrollTopOnHide: boolean }) => void
+            onEditGridClick: '&', // () => void
+            onAddElementClick: '&' // () => void
         },
         templateUrl: '/static/dataiku/js/dashboards/components/dashboard-grid/dashboard-grid.component.html',
-        controller: function(DashboardFilters, $timeout, $rootScope, DashboardUtils, DashboardPageUtils, TileLoadingState, TileLoadingBehavior, $element, $scope, $q, ContextualMenu, TileUtils, LinkedList, DetectUtils, Debounce, WT1, Logger) {
+        controller: function(DashboardFilters, $timeout, $rootScope, DashboardUtils, DashboardPageUtils, TileLoadingState, TileLoadingBehavior, $element, $scope, $q, ContextualMenu, TileUtils, LinkedList, DetectUtils, Debounce, WT1, Logger, translate) {
             const ctrl = this;
 
             /*
@@ -56,6 +59,10 @@
             ctrl.gridMinHeight = null;
             ctrl.TileUtils = TileUtils;
             ctrl.tileMinSizeByTileId = {};
+            ctrl.translate = translate;
+            ctrl.editButtonLabel = translate('DASHBOARD.EDIT.BUTTON', 'Edit');
+            ctrl.emptyStateViewModeDescription = translate('DASHBOARD.PAGE.EMPTY_STATE.VIEW.DESCRIPTION','Switch to edit mode to add tiles.');
+
 
             let isTryingToDragLockedTile = false;
             let blockTileDeselection;
@@ -655,10 +662,12 @@
 
                         updateTilesPosition(itemById);
                         const tileIdsToRemoveSet = getTileIdsToRemoveSet(itemById);
-                        cleanIndexesFromRemovedTiles(tileIdsToRemoveSet);
+                        cleanRemovedTilesFromHookIndexes(tileIdsToRemoveSet);
                         const remainingTiles = ctrl.tiles.filter(tile => !tileIdsToRemoveSet.has(tile.$tileId));
                         ctrl.tilesChange({ $tiles: remainingTiles });
                         $scope.$emit('dashboardSyncModelsDone');
+                    } else {
+                        cleanHookIndexes();
                     }
                 });
             };
@@ -690,7 +699,7 @@
                 return tileIdsToRemove;
             }
 
-            function cleanIndexesFromRemovedTiles(tileIdsToRemoveSet) {
+            function cleanRemovedTilesFromHookIndexes(tileIdsToRemoveSet) {
                 tileIdsToRemoveSet.forEach((tileId) => {
                     initializedTileIds.delete(tileId);
                     delete ctrl.hook.loadPromises[tileId];
@@ -701,6 +710,27 @@
                 });
             }
 
+            function cleanHookIndexes() {
+                if (ctrl.hook == null) {
+                    return;
+                }
+                /*
+                 * Manually clear hook indexes to help the garbage collector.
+                 *
+                 * Tiles and the grid keep circular references through ctrl.hook.
+                 * Combined with pending $timeout callbacks, this can prevent
+                 * proper cleanup after the component is destroyed.
+                 *
+                 * To avoid leaks, we explicitly reset these collections (assigning {})
+                 * in case any child tile still holds a reference to them.
+                 */
+                ctrl.hook.loadPromises = {};
+                ctrl.hook.reloadPromises = {};
+                ctrl.hook.loadStates = {};
+                ctrl.hook.visibleStates = {};
+                ctrl.hook.adjacentStates = {};
+            }
+
             function updateSelectedTilesResizeHandles() {
                 selectedTileIds.forEach(tileId => {
                     const tileElement = DashboardPageUtils.getTileElementFromTileId($element, tileId);
@@ -882,8 +912,14 @@
                 if (scrollingContainer != null) {
                     scrollingContainer.removeEventListener('wheel', handleShiftWheelScroll, { passive: false });
                 }
+                cleanHookIndexes();
+            });
+
+            $element.on('$destroy', function() {
                 if (ctrl.gridListRendered) {
                     grid.destroy();
+                    ctrl.gridListRendered = false;
+                    grid = null;
                 }
             });
 
@@ -967,35 +1003,39 @@
                     }
                     DashboardPageUtils.positionResizeHandles(tileWrapper, ctrl.tileSpacing);
                 });
-                return $timeout(() => {
-                    hasUsedMultiSelectKey = isMultiSelectKeyPressed;
-                    const tilesById = getTilesById();
-                    if (tiles.length === 1) {
-                        if (isMultiSelectKeyPressed) {
-                            // it might unselect tile if already selected
-                            if (selectedTileIds.has(tiles[0].$tileId)) {
-                                selectedTileIds.delete(tiles[0].$tileId);
-                            } else {
-                                selectedTileIds.add(tiles[0].$tileId);
-                            }
+                hasUsedMultiSelectKey = isMultiSelectKeyPressed;
+                const tilesById = getTilesById();
+                if (tiles.length === 1) {
+                    if (isMultiSelectKeyPressed) {
+                        // it might unselect tile if already selected
+                        if (selectedTileIds.has(tiles[0].$tileId)) {
+                            selectedTileIds.delete(tiles[0].$tileId);
                         } else {
-                            // Here one single tile is selected by a single click
-                            hasUsedRectangleSelection = false;
-                            selectedTileIds.clear();
                             selectedTileIds.add(tiles[0].$tileId);
                         }
-
-                        if (selectedTileIds.size === 0) {
-                            $rootScope.$emit('dashboardUnselectTiles');
-                        } else {
-                            $rootScope.$emit('dashboardSelectTiles', { tiles: Array.from(selectedTileIds).map(id => tilesById[id]), grid });
-                        }
                     } else {
+                        // Here one single tile is selected by a single click
+                        hasUsedRectangleSelection = false;
                         selectedTileIds.clear();
-                        tiles.forEach(tile => selectedTileIds.add(tile.$tileId));
-                        $rootScope.$emit('dashboardSelectTiles', { tiles, grid });
+                        selectedTileIds.add(tiles[0].$tileId);
                     }
-                });
+                    if (selectedTileIds.size === 0) {
+                        return $timeout(() => {
+                            $rootScope.$emit('dashboardUnselectTiles');
+                        });
+                    } else {
+                        const selectedTiles = Array.from(selectedTileIds).map(id => tilesById[id]);
+                        return $timeout(() => {
+                            $rootScope.$emit('dashboardSelectTiles', { tiles: selectedTiles, grid });
+                        });
+                    }
+                } else {
+                    selectedTileIds.clear();
+                    tiles.forEach(tile => selectedTileIds.add(tile.$tileId));
+                    return $timeout(() => {
+                        $rootScope.$emit('dashboardSelectTiles', { tiles, grid });
+                    });
+                }
             };
 
             function deselectTiles() {
