@@ -6,7 +6,7 @@
     /**
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/edit.js
      */
-    app.controller('NewDashboardModalController', function($scope, $stateParams, $rootScope, DataikuAPI, DashboardUtils, DSSVisualizationThemeUtils) {
+    app.controller('NewDashboardModalController', function($scope, $stateParams, $rootScope, DataikuAPI, DashboardUtils, DSSVisualizationThemeUtils, WT1) {
         $scope.dashboard = {
             projectKey: $stateParams.projectKey,
             owner: $scope.appConfig.user.login,
@@ -24,6 +24,7 @@
             DataikuAPI.dashboards.save($scope.dashboard).success(function(dashboard) {
                 $scope.dashboard = dashboard;
                 $scope.resolveModal($scope.dashboard);
+                WT1.tryEvent('dashboard-create', () => ({ source: 'dashboards-list' }));
             }).error(setErrorInScope.bind($scope));
         };
     });
