@@ -11,6 +11,8 @@
     app.constant('ChartInsightHandler', {
         name: 'Chart',
         desc: 'Visualize data from your source',
+        i18nNameKey: 'INSIGHTS.CHART.NAME',
+        i18nDescKey: 'INSIGHTS.CHART.DESC',
         icon: 'icon-dku-nav_dashboard',
         color: 'chart',
 
@@ -29,6 +31,7 @@
                 showYAxis: true,
                 showLegend: false,
                 showBrush: false,
+                showBreadcrumb: true,
                 inheritLegendPlacement: true,
                 showTooltips: true,
                 autoPlayAnimation: true,
@@ -44,8 +47,8 @@
         }
     });
 
-    app.directive('chartInsightTile', function($controller, ChartRequestComputer, MonoFuture, ChartTypeChangeHandler, ChartFeatures, ChartDataUtils, DashboardUtils, DashboardPageUtils,
-        TileLoadingState, TileLoadingBehavior, DashboardFilters, Logger, $stateParams, $q, ChartStoreFactory, DSSVisualizationThemeUtils) {
+    app.directive('chartInsightTile', function($controller, ChartRequestComputer, MonoFuture, ChartTypeChangeHandler, ChartFeatures, ChartDataUtils, DashboardUtils, DashboardPageUtils, ChartDrilldown,
+        TileLoadingState, TileLoadingBehavior, DashboardFilters, Logger, $stateParams, $q, ChartStoreFactory, DSSVisualizationThemeUtils, FilterFacetsService, ChartFilters, translate, ChartHierarchyDimension) {
         return {
             templateUrl: '/templates/dashboards/insights/chart/chart_tile.html',
             scope: {
@@ -61,7 +64,7 @@
                 let originChartDef;
 
                 $scope.DashboardUtils = DashboardUtils;
-                $scope.insight.$filteringStatus = {};
+                $scope.insight.$tileFilteringInformation = {};
                 $scope.originDashboard = {
                     id: $stateParams.dashboardId,
                     name: $stateParams.dashboardName,
@@ -108,7 +111,28 @@
                 let unregisterLoadingStateWatcher;
                 let unregisterVisibilityWatcher;
                 let unregisterFiltersWatcher;
+                let hasCaseInsensitiveColumnNames = false;
+                let isChartFixedUp = false;
 
+                $scope.breadcrumbs = ChartDrilldown.getBreadcrumbs($scope.insight.params.def, $scope.filters);
+                const chartHierarchies = ChartHierarchyDimension.getHierarchyDimensionsForChartType($scope.insight.params.def);
+                for (const hierarchy of chartHierarchies) {
+                    $scope.$watch(() => hierarchy.level, (nv, ov) => {
+                        if (nv == null || nv === ov) {
+                            return;
+                        }
+                        $scope.breadcrumbs = ChartDrilldown.getBreadcrumbs($scope.insight.params.def, $scope.filters);
+
+                        const loadingData = {
+                            successRoutine: () => {
+                            },
+                            errorRoutine: DashboardUtils.setError.bind([$scope]),
+                            unconfiguredRoutine: DashboardUtils.setLoaded.bind([$scope])
+                        };
+
+                        return computeAndExecutePivotRequest($scope.insight.params.def, loadingData, true);
+                    });
+                }
 
                 $scope.$on('$destroy', function() {
                     $(window).off('resize', broadcastResize);
@@ -131,28 +155,6 @@
                     loadReject = reject;
                     errorRoutine = DashboardUtils.setError.bind([$scope, reject]);
 
-                    const unregister = $scope.$watch('chart.summary', nv => {
-                        if (!nv) {
-                            return;
-                        }
-                        unregister();
-                        if (!$scope.insight.$fixedUp) {
-                            $scope.insight.params.summary = nv;
-                            const insightTheme = getTheme($scope.insight);
-                            ChartTypeChangeHandler.fixupChart($scope.insight.params.def, insightTheme);
-                            ChartTypeChangeHandler.fixupSpec($scope.insight.params, insightTheme);
-
-                            if ($scope.origInsight) {
-                                $scope.origInsight.params.summary = nv;
-                                const origInsightTheme = getTheme($scope.origInsight);
-                                ChartTypeChangeHandler.fixupChart($scope.origInsight.params.def, origInsightTheme);
-                                ChartTypeChangeHandler.fixupSpec($scope.origInsight.params, origInsightTheme);
-                            }
-                        }
-                    });
-
-                    $scope.loading = true;
-
                     const loadingData = {
                         successRoutine: DashboardUtils.setLoaded.bind([$scope, resolve]),
                         errorRoutine: errorRoutine,
@@ -161,12 +163,47 @@
                         reject: reject
                     };
 
-                    unregisterFiltersWatcher = $scope.$watch('filters', () => {
-                        applySlideFilters(loadingData);
-                    });
+                    $scope.loading = true;
+                    DashboardFilters.hasCaseInsensitiveColumnNames($scope.resolvedDataset.projectKey, $scope.resolvedDataset.datasetName, $stateParams.projectKey)
+                        .then(hasCaseInsensitiveColumnNamesStatus => {
+                            hasCaseInsensitiveColumnNames = hasCaseInsensitiveColumnNamesStatus;
+
+                            if ($scope.globalFiltersData.usableColumns == null) {
+                                return $scope.fetchColumnsSummary($scope.insight.projectKey)
+                                    .then(() => {
+                                        $scope.globalFiltersData.usableColumns = $scope.usableColumns;
+                                    });
+                            }
+                        }).then(() => {
+                            if (!isChartFixedUp) {
+                                fixUpChartAndSetSummary($scope.insight, $scope.chart.summary);
+                                if ($scope.origInsight) {
+                                    fixUpChartAndSetSummary($scope.origInsight, $scope.chart.summary);
+                                }
+                                isChartFixedUp = true;
+                            }
+                            return loadChart(loadingData);
+                        }).then(() => {
+                            if (unregisterFiltersWatcher) {
+                                unregisterFiltersWatcher();
+                            }
+                            unregisterFiltersWatcher = $scope.$watch('filters', (nv, ov) => {
+                                // Avoid initialization call as it is already called above
+                                if (nv === ov) {
+                                    return;
+                                }
+                                if (chartHierarchies && chartHierarchies.length) {
+                                    $scope.breadcrumbs = ChartDrilldown.getBreadcrumbs($scope.insight.params.def, nv);
+                                }
+                                loadChart(loadingData);
+                            });
+                        }).catch((error) => {
+                            handleLoadingFailure( loadingData, error );
+                        });
 
                     return TileLoadingBehavior.DELAYED_COMPLETE;
                 };
+
                 $scope.hook.loadPromises[$scope.tile.$tileId] = $scope.load;
                 $scope.hook.reloadPromises[$scope.tile.$tileId] = $scope.load;
 
@@ -195,6 +232,12 @@
                     executeFiltersRequest($scope.filters, additionalRequestCallbacks);
                 };
 
+                function fixUpChartAndSetSummary(chartInsight, summary) {
+                    chartInsight.params.summary = summary;
+                    const theme = getTheme(chartInsight);
+                    ChartTypeChangeHandler.fixupChart(chartInsight.params.def, theme);
+                    ChartTypeChangeHandler.fixupSpec(chartInsight.params, theme);
+                }
 
                 /**
                  * To handle the failure of the loading routine
@@ -213,6 +256,19 @@
                  * @param {Function}        error.statusText
                  */
                 const handleLoadingFailure = ( loadingData, error ) => {
+                    if ($scope.unconfigured || $scope.tile.$allFilteredOut) {
+                        return;
+                    }
+                    // Handle possible client side error to avoid tile being stuck in loading state
+                    if (!_.isObject(error) || !error.data || !error.data.detailedMessage) {
+                        Logger.error(`Failed to load insight ${$scope.insight.id}`, error);
+                        error = {
+                            data:{
+                                detailedMessage: translate('DASHBOARD.FILTERABLE_INSIGHTS.ERRORS.LOADING_FAILED', 'error occurred while loading insight')
+                            }
+                        };
+                    }
+
                     const { data, status, headers, config, statusText } = error;
                     if (loadingData) {
                         loadingData.errorRoutine && loadingData.errorRoutine(data, status, headers, config, statusText);
@@ -276,6 +332,12 @@
                         };
 
                         request = ChartRequestComputer.compute(chartDef, element.width(), element.height(), $scope.chartSpecific);
+
+                        if($scope.originDashboard) {
+                            // set allValuesInSample for each filter for easier tooltip calculation
+                            request.filters.forEach(filter => filter.allValuesInSample = true);
+                        }
+
                         const validity = ChartTypeChangeHandler.getValidity({ def: chartDef });
                         if (loadingData && validity.valid === false) {
                             loadingData.errorRoutine({ detailedMessage: 'Invalid configuration, ' + validity.message });
@@ -307,7 +369,7 @@
                             if (hasGlobalFilters) {
                                 $scope.tile.$allFilteredOut = false;
                             }
-                            const sampleUIData = ChartDataUtils.getSampleMetadataAndSummaryMessage(data.result.pivotResponse, 'Click to edit the insight');
+                            const sampleUIData = ChartDataUtils.getSampleMetadataAndSummaryMessage(data.result.pivotResponse, $scope.insight.params.def.type, translate('DASHBOARD.TILES.HEADER.TOOLTIP.CLICK_TO_EDIT', 'Click to edit insight'));
                             $scope.tile.$sampleMetadata = sampleUIData.sampleMetadata;
                             $scope.tile.$samplingSummaryMessage = sampleUIData.summaryMessage;
                             $scope.tile.$clickableSamplingSummaryMessage = sampleUIData.clickableSummaryMessage;
@@ -320,6 +382,8 @@
                                 // We still need to display the insight to say it's been filtered out with a proper empty state.
                                 handleLoadingSuccess(loadingData, $scope.loadedCallback());
                                 $scope.tile.$allFilteredOut = true;
+                                // resend the request to get filters facets & display them in the tooltip
+                                getChartFiltersFacets();
                             } else if (data.code === 'CANNOT_DISPLAY_WITH_EMPTY_AXES') {
                                 handleLoadingSuccess(loadingData, $scope.loadedCallback());
                                 $scope.tile.$display0Warning = true;
@@ -340,7 +404,7 @@
                     unregisterLoadingStateWatcher = $scope.$watch('loaded', (nv) => {
                         if (nv === true) {
                             unregisterLoadingStateWatcher(); // remove watch
-                            applySlideFilters();
+                            loadChart();
                         }
                     });
                 }
@@ -353,7 +417,7 @@
                         if (nv === true) {
                             unregisterVisibilityWatcher(); // remove watch
                             unregisterVisibilityWatcher = null;
-                            applySlideFilters();
+                            loadChart();
                         }
                     });
                 }
@@ -368,38 +432,27 @@
                  * @param {Function}    loadingData.reject
                  * @param {boolean}     [forceFilterableTilesRefresh=false]
                  */
-                function applySlideFilters(loadingData) {
+                function loadChart(loadingData) {
                     /*
                      * If the chart is not yet loaded and no loading instructions are provided,
                      * wait for it to complete before applying latest filter changes (which happened in the meantime)
                      */
                     if (!loadingData && !$scope.loaded && !unregisterLoadingStateWatcher) {
                         watchLoadingStateChange();
-                        return;
+                        return Promise.resolve();
                     }
 
+                    /* If the tile is not visible nor adjacent, wait for it to become visible before loading the chart */
                     if (!DashboardPageUtils.isTileVisibleOrAdjacent($scope.tile.$tileId, $scope.hook)) {
                         watchVisibilityChange();
-                        return;
+                        return Promise.resolve();
                     }
-
-                    DashboardFilters.hasCaseInsensitiveColumnNames($scope.resolvedDataset.projectKey, $scope.resolvedDataset.datasetName, $stateParams.projectKey).then(hasCaseInsensitiveColumnNames => {
-
-                        if ($scope.globalFiltersData.usableColumns == null) {
-                            $scope.fetchColumnsSummary($scope.insight.projectKey)
-                                .then(() => {
-                                    $scope.globalFiltersData.usableColumns = $scope.usableColumns;
-                                    return DashboardFilters.applyFilters($scope.filters, $scope.filtersParams, $scope.globalFiltersData, loadingData, hasCaseInsensitiveColumnNames);
-                                }).catch(( error ) => {
-                                    handleLoadingFailure( loadingData, error );
-                                });
-                        } else {
-                            DashboardFilters.applyFilters($scope.filters, $scope.filtersParams, $scope.globalFiltersData, loadingData, hasCaseInsensitiveColumnNames);
-                        }
-                    }).catch((error) => {
-                        handleLoadingFailure( loadingData, error );
-                    });
-                };
+                    // It actually trigger the chart loading even if there is no filter to apply
+                    return DashboardFilters.applyFilters($scope.filters, $scope.filtersParams, $scope.globalFiltersData, loadingData, hasCaseInsensitiveColumnNames)
+                        .catch((error) => {
+                            handleLoadingFailure( loadingData, error );
+                        });
+                }
 
                 function executeFiltersRequest(filters, loadingData) {
                     const chartDef = { ...$scope.insight.params.def };
@@ -420,15 +473,15 @@
                     usableColumns: $scope.usableColumns,
                     applyFilters: executeFiltersRequest,
                     parseResponse: (data) => data.data.result.pivotResponse,
-                    filteringStatus: angular.copy($scope.insight.$filteringStatus),
+                    tileFilteringInformation: angular.copy($scope.insight.$tileFilteringInformation),
                     sourceDataset: $scope.insight.params.datasetSmartName,
                     sampleSettings: $scope.insight.params.refreshableSelection,
                     pageId: $stateParams.pageId
                 };
 
-                // globalFiltersData.filteringStatus is mutated by applyFilters, and we need to update insight.$filteringStatus reference to trigger Angular's digest cycle.
-                $scope.$watch('globalFiltersData.filteringStatus', filteringStatus => {
-                    $scope.insight.$filteringStatus = angular.copy(filteringStatus);
+                // globalFiltersData.tileFilteringInformation is mutated by applyFilters, and we need to update insight.$tileFilteringInformation reference to trigger Angular's digest cycle.
+                $scope.$watch('globalFiltersData.tileFilteringInformation', tileFilteringInformation => {
+                    $scope.insight.$tileFilteringInformation = angular.copy(tileFilteringInformation);
                 }, true);
 
                 if ($scope.tile.autoLoad) {
@@ -498,6 +551,7 @@
                     $scope.noXAxis = !nv.showXAxis;
                     $scope.noYAxis = !nv.showYAxis;
                     $scope.noTooltips = !nv.showTooltips;
+                    $scope.noBreadcrumb = !nv.showBreadcrumb;
                     $scope.autoPlayAnimation = nv.autoPlayAnimation;
 
                     if ($scope.chart && (isFirstChange || nv.useInsightTheme !== ov.useInsightTheme)) {
@@ -587,11 +641,26 @@
 
                     return originChartTheme;
                 }
+
+                function getChartFiltersFacets() {
+                    // we resend the same filters to get the correct facets
+                    const { store } = ChartStoreFactory.getOrCreate($scope.chart.def.$chartStoreId);
+                    const request = ChartFilters.buildFiltersRequest([...$scope.chart.def.filters, ...store.getAppliedDashboardFilters()], {});
+                    return FilterFacetsService.getFilterFacets(
+                        $scope.insight.projectKey, $scope.getDataSpec(), request, $scope.chart.summary?.requiredSampleId, MonoFuture()
+                    )
+                        .then((data) => {
+                            $scope.globalFiltersData.tileFilteringInformation.allFiltersFacets = data.data.result.pivotResponse.filterFacets;
+                            $scope.insight.$tileFilteringInformation = angular.copy($scope.globalFiltersData.tileFilteringInformation);
+                            DashboardFilters.setAllInsightFiltersFacets(data.data.result.pivotResponse.filterFacets, $scope.insight.id);
+                            DashboardFilters.setInsightsFiltersFacets(data.data.result.pivotResponse.filterFacets, $scope.insight.id);
+                        });
+                }
             }
         };
     });
 
-    app.directive('chartInsightTileParams', function(ChartFeatures){
+    app.directive('chartInsightTileParams', function(ChartFeatures, ChartHierarchyDimension){
         return {
             templateUrl: '/templates/dashboards/insights/chart/chart_tile_params.html',
             scope: {
@@ -604,8 +673,12 @@
                     $scope.noTooltips = !ChartFeatures.canShowTooltips(type);
                 });
 
-                $scope.$watch('insight.params.def', (def) =>
-                    $scope.canDisplayBrush = ChartFeatures.canDisplayBrush(def));
+                $scope.$watch('insight.params.def', (def) => {
+                    $scope.canDisplayBrush = ChartFeatures.canDisplayBrush(def);
+
+                    const hierarchiesInChart = ChartHierarchyDimension.getHierarchyDimensionsForChartType(def);
+                    $scope.canDisplayBreadcrumb = ChartFeatures.canDisplayBreadcrumb(def.type) && !!hierarchiesInChart.length;
+                });
 
                 $scope.$watch('insight.params.def.animationDimension', (animationDimension) =>
                     $scope.noAnimation = (animationDimension || []).length === 0);
@@ -616,7 +689,7 @@
         };
     });
 
-    app.directive('chartInsightCreateForm', function(DataikuAPI, ChartTypeChangeHandler, DatasetChartsUtils, DSSVisualizationThemeUtils) {
+    app.directive('chartInsightCreateForm', function(DataikuAPI, ChartTypeChangeHandler, DatasetChartsUtils, DSSVisualizationThemeUtils, translate) {
         return {
             templateUrl: '/templates/dashboards/insights/chart/chart_create_form.html',
             scope: true,
@@ -637,12 +710,16 @@
                         });
                 };
 
-                $scope.hook.defaultName = 'Chart';
+                $scope.hook.defaultName = translate('INSIGHTS.CHART.NAME', 'Chart');
                 $scope.$watch('hook.sourceObject', function(nv) {
                     if (!nv || !nv.label) {
                         return;
                     }
-                    $scope.hook.defaultName = 'Chart on ' + nv.label;
+                    $scope.hook.defaultName = translate(
+                        'INSIGHTS.CHART.NAME_ON_SOURCE',
+                        'Chart on ' + nv.label,
+                        { source: nv.label }
+                    );
                 });
             }
         };
@@ -733,17 +810,22 @@
             });
         }
 
-        $rootScope.$on('chartSamplingChanged', function(event, opts) {
+        const unregisterChartSamplingChangedEvent = $rootScope.$on('chartSamplingChanged', function(event, opts) {
             if (angular.equals($scope.chart, opts.chart)) {
                 $scope.chart.summary = null;
                 fetchSummaryAndExecute(true);
             }
         });
 
+        $scope.$on('$destroy', () => {
+            unregisterChartSamplingChangedEvent();
+        });
+
         // chartHandler options
         $scope.noThumbnail = true;
         $scope.addCustomMeasuresToScopeAndCache($scope.insight.params.customMeasures);
         $scope.addBinnedDimensionToScopeAndCache($scope.insight.params.reusableDimensions);
+        $scope.addHierarchiesToScopeAndCache($scope.insight.params.hierarchies);
     });
 
     app.directive('chartInsightView', function($controller, translate) {
