@@ -30,7 +30,18 @@ app.controller('PublishIPythonController', function ($scope, $stateParams, Datai
     };
     $scope.loggedIn = $scope.appConfig.loggedIn;
     $scope.sessionExpired = false;
-    $scope.exportCreateAndPinInsight = function(notebook) {
+
+    /**
+     * Export notebook to dashboard and optionally save it before export.
+     *
+     * Warning: Implicit assumption is that a notebook can only be saved if opened on the current page, as logic will
+     * look for a specific DOM element for saving.
+     */
+    $scope.exportCreateAndPinInsight = function(notebook, saveBeforeExport) {
+        if (!saveBeforeExport) {
+            createAndPinInsight(notebook);
+            return;
+        }
         $('iframe#jupyter-iframe')[0].contentWindow.IPython.notebook.save_notebook().then(function() {
             createAndPinInsight(notebook);
         }).catch(setJupyterErrorInScope($scope));
