@@ -840,7 +840,12 @@
 
                     scope.craftBinningFormChart = function(params) {
                         $rootScope.globallyOpenContextualMenu = undefined; // close currently opened one
-                        scope.$emit('craftBinningFormChart', { dimension: angular.copy(params.dimension), dimensionRef: params.dimension, isEditMode: params.isEditMode, hideOneTickPerBin: params.hideOneTickPerBin });
+                        scope.$emit('craftBinningFormChart', { dimension: angular.copy(params.dimension), dimensionRef: params.dimension, isEditMode: params.isEditMode, hideOneTickPerBin: params.hideOneTickPerBin, customBinningOnly: params.customBinningOnly });
+                    };
+
+                    scope.createReusableDimensionFromChart = function(params) {
+                        $rootScope.globallyOpenContextualMenu = undefined; // close currently opened one
+                        scope.$emit('createReusableDimensionFromChart', { ...params, dimension: angular.copy(params.dimension), dimensionRef: params.dimension });
                     };
 
                     /*
