@@ -679,7 +679,7 @@ app.directive('achievements', function(AchievementService, Notification, Logger,
     return {
         restrict : 'A',
         link : function($scope, element, attrs) {
-            Notification.registerEvent("achievement-unlocked", function(evt, message) {
+            const achievementUnlockedUnsubscribe = Notification.registerEvent("achievement-unlocked", function(evt, message) {
                 Logger.info("Achievement unlocked", message);
 
                 $http.get(templatePath, {cache: $templateCache}).then(function(response) {
@@ -699,6 +699,7 @@ app.directive('achievements', function(AchievementService, Notification, Logger,
                     window.setTimeout(function() {a.addClass("gone");}, 5000);
                 });
             });
+            $scope.$on('$destroy', achievementUnlockedUnsubscribe);
         }
     };
 });
