@@ -5,21 +5,26 @@
         bindings: {
             initialMessages: '<',
             lastMessageId: '<',
+            context: '<',
             sessionId: '<',
             availableLlms: '<',
             hasActiveLlm: '<',
             warnAboutRaLlmSearchMode: '<',
             activeLlmIcon: '<',
             emptyStateMessage: '<',
+            customAssistantMessageSettings: '<?',
+            showContextSelector: '<?',
+            settingsComparator: '&?',
             onBeforeSendMessage: '&?',
             onSendMessage: '&',
             onForkMessage: '&?',
             onStopStreaming: '&',
             onChatCompletion: '&',
+            onSetContext: '&',
             processResponse: '&?'
         },
         templateUrl: '/static/dataiku/shared/components/prompt-chat/prompt-chat.component.html',
-        controller: function($rootScope, $scope, $timeout, PromptUtils, PromptChatService, TraceExplorerService, ClipboardUtils, Logger, $dkuSanitize) {
+        controller: function($rootScope, $scope, $timeout, PromptUtils, PromptChatService, TraceExplorerService, ClipboardUtils, Logger, CreateModalFromTemplate, CodeMirrorSettingService, $dkuSanitize) {
             const $ctrl = this;
             const entryElId = 'prompt-chat-entry';
 
@@ -30,7 +35,7 @@
             $ctrl.iconSize = 16;
             $ctrl.editedMessage = '';
             $ctrl.editedMessageId = '';
-            
+
             $ctrl.$onChanges = (changes) => {
                 // don't reload the messages if the active session is still streaming
                 if (changes.initialMessages) {
@@ -59,10 +64,13 @@
                     ...($ctrl.chatSessions[activeSessionId] || {}),
                     streamedMessage: '',
                     rawMessage: '',
+                    rawReasoning: '',
+                    reasoning: '',
                     currentlyStreaming: true,
                     useMarkdown: true,
                     streamingSupported: true,
                     streamingNotSupportedMessage: '',
+                    lastCustomAssistantMessageSettings: angular.copy($ctrl.customAssistantMessageSettings),
                     abortController: new AbortController(),
                     runData,
                     isRerun,
@@ -72,6 +80,8 @@
                 const CHAR_LIMIT_FOR_MARKDOWN = 500;
                 const THROTTLE_MIN_LENGTH = 10000;
                 let previousLength = 0;
+                let currentReasoningIndex;
+                let currentReasoningId;
                 const callback = chunk => {
                     scrollChat(activeSessionId, false);
 
@@ -79,6 +89,49 @@
                     const data = chunk['data'];
                     switch(event) {
                         case 'completion-chunk': {
+                            if (data.artifacts && data.artifacts.length) {
+                                data.artifacts.forEach((artifact) => {
+                                    if (artifact.type === 'REASONING' && artifact.parts  && artifact.parts.length) {
+                                        if (currentReasoningId == null){
+                                            currentReasoningId = artifact.id;
+                                        }
+                                        if (currentReasoningId !== artifact.id) {
+                                            activeSession.rawReasoning += "\n\n";
+                                            currentReasoningId = artifact.id;
+                                        }
+                                        artifact.parts.forEach((artifactPart) => {
+                                            if (artifactPart.type === 'TEXT') {
+                                                if (currentReasoningIndex == null) {
+                                                    currentReasoningIndex = artifactPart.index;
+                                                }
+                                                if (currentReasoningIndex !== artifactPart.index) {
+                                                    currentReasoningIndex = artifactPart.index;
+                                                    activeSession.rawReasoning += "\n\n";
+                                                }
+                                                activeSession.rawReasoning += artifactPart.text;
+
+                                                // throttle text streaming if the string is too long
+                                                if (activeSession.useMarkdown) {
+                                                    // if there is an error parsing the HTML (e.g., LLM is spitting out garbage text), stop using markdown
+                                                    try {
+                                                        activeSession.reasoning = $dkuSanitize(marked(activeSession.rawReasoning, {
+                                                            breaks: true
+                                                        }));
+                                                    } catch(e) {
+                                                        Logger.error('Error parsing markdown HTML, switching to plain text', e);
+                                                        activeSession.useMarkdown = false;
+                                                    }
+                                                }
+                                                if (!activeSession.useMarkdown) {
+                                                    activeSession.reasoning = activeSession.rawReasoning;
+                                                }
+                                                $scope.$apply();
+                                                scrollChat(activeSessionId, false);
+                                            }
+                                        })
+                                    }
+                                })
+                            }
                             if (data.text) {
                                 const text = data.text;
                                 activeSession.rawMessage += text;
@@ -91,7 +144,6 @@
                                         activeSession.streamedMessage = $dkuSanitize(marked(activeSession.rawMessage, {
                                             breaks: true
                                         }));
-                                        $scope.$apply();
                                         previousLength = currentLength;
                                     } catch(e) {
                                         Logger.error('Error parsing markdown HTML, switching to plain text', e);
@@ -101,9 +153,8 @@
 
                                 if (!activeSession.useMarkdown) {
                                     activeSession.streamedMessage = activeSession.rawMessage;
-                                    $scope.$apply();
                                 }
-
+                                $scope.$apply();
                                 scrollChat(activeSessionId, false);
                             }
                             break;
@@ -116,7 +167,7 @@
                             if ($ctrl.sessionId === activeSessionId) {
                                 onChatCompletion($ctrl.processResponse ? $ctrl.processResponse({ response: data }) : data, activeSessionId);
                                 $timeout(() => {
-                                    scrollChat(activeSessionId, false);
+                                    scrollChat(activeSessionId, true);
                                 });
                             }
                             break;
@@ -156,12 +207,13 @@
                     defaultMessage: '' // reset default message if applicable
                 };
                 const { callback, activeSession } = getChatHandler(chatHandlerParams, isRerun);
-                $ctrl.onSendMessage({ newMessage, abortController: activeSession.abortController, messages: $ctrl.messages, callback });
+                $ctrl.onSendMessage({ newMessage, abortController: activeSession.abortController, messages: PromptChatService.cleanChatMesssages($ctrl.messages), callback });
             }
+
             // used for auto textarea resizing
             function setMessageField(el) {
                 el.parentNode.dataset.replicatedValue = el.value;
-            };
+            }
 
             $ctrl.sendMessage = () => {
                 if ($ctrl.onBeforeSendMessage) {
@@ -205,7 +257,7 @@
                         content : $ctrl.editedMessage
                     }
                 };
-                $ctrl.messageBranch = PromptChatService.buildCurrentChatBranch($ctrl.messages, parentId);
+                $ctrl.messageBranch = buildBranch(parentId);
                 $ctrl.messageBranch.push(newMessage);
                 $ctrl.streamParentId = parentId;
                 onSendMessage(newMessage, null, false, true);
@@ -229,7 +281,7 @@
             $ctrl.rerunResponse = (message) => {
                 const parentId = message.parentId;
                 const newMessage = $ctrl.messages[parentId];
-                $ctrl.messageBranch = PromptChatService.buildCurrentChatBranch($ctrl.messages, parentId);
+                $ctrl.messageBranch = buildBranch(parentId);
                 onSendMessage(newMessage, { version: message.version + 1, parentId }, true);
             };
 
@@ -269,7 +321,7 @@
                     nextMessage = $ctrl.messages[nextMessage.childrenIds[0]];
                 }
                 $ctrl.lastMessageId = nextMessage.id;
-                $ctrl.messageBranch = PromptChatService.buildCurrentChatBranch($ctrl.messages, $ctrl.lastMessageId);
+                $ctrl.messageBranch = buildBranch($ctrl.lastMessageId);
             };
 
             $ctrl.onKeydown = (event, sendEdited, parentId) => {
@@ -319,6 +371,39 @@
                 TraceExplorerService.openTraceExplorer(message.fullTrace);
             };
 
+            $ctrl.openEditContextModal = () => {
+                CreateModalFromTemplate("/templates/widgets/edit-context-modal.html", $scope, null, function(modalScope) {
+                    modalScope.codeMirrorSettings = {
+                        ...CodeMirrorSettingService.get('application/json', {
+                            onLoad: editor => {
+                                $timeout(function() {
+                                    editor.focus();
+                                }, 50);
+                            }
+                        }),
+                    };
+
+                    modalScope.uiState = modalScope.uiState || {};
+                    modalScope.uiState.showContextSelector = $ctrl.showContextSelector;
+                    modalScope.uiState.contextWrapper = {
+                        context: $ctrl.context ? angular.copy($ctrl.context) : {}
+                    };
+                    modalScope.setTemplate = (template) => {
+                        modalScope.uiState.contextWrapper = {
+                            context: template
+                        };
+                    };
+                    modalScope.confirm = () => {
+                        // add context to messages
+                        $ctrl.context = modalScope.uiState.contextWrapper?.context;
+                        if ($ctrl.onSetContext) {
+                            $ctrl.onSetContext({ context: $ctrl.context });
+                        }
+                        modalScope.dismiss();
+                    };
+                });
+            };
+
             $ctrl.stopStreaming = function() {
                 const sessionId = $ctrl.sessionId;
                 $ctrl.chatSessions[sessionId]?.abortController.abort();
@@ -341,26 +426,25 @@
             };
 
             // Called if prompt chat kernel is manually stopped outside of prompt chat component
-            $rootScope.$on('llmStopDevKernel', (_, sessionId) => {
+            const llmStopDevKernelUnsubscribe = $rootScope.$on('llmStopDevKernel', (_, sessionId) => {
                 if ($ctrl.chatSessions[sessionId]?.currentlyStreaming) {
                     $ctrl.stopStreaming();
                 }
             });
-
-            $ctrl.$onDestroy = () => {
-                PromptChatService.clearLogs();
-            };
+            $scope.$on('$destroy', llmStopDevKernelUnsubscribe);
 
             function onChatCompletion(response) {
                 $ctrl.lastMessageId = response.lastMessageId;
-                prepareMessages(response.chatMessages, response.lastMessageId);
+                const customAssistantMessageSettings = $ctrl.chatSessions[$ctrl.sessionId]?.lastCustomAssistantMessageSettings;
+                prepareMessages(response.chatMessages, response.lastMessageId, customAssistantMessageSettings);
                 if (response.log) {
                     PromptChatService.setLog($ctrl.sessionId, response.log);
                 }
 
                 $ctrl.onChatCompletion({ 
                     lastMessageId: response.lastMessageId,
-                    messages: response.chatMessages 
+                    messages: response.chatMessages,
+                    enrichedMessages: $ctrl.messages
                 });
             }
 
@@ -390,9 +474,17 @@
                 }
             }
 
-            function prepareMessages(messages, lastMessageId) {
-                $ctrl.messages = PromptChatService.enrichChatMessages(messages);
-                $ctrl.messageBranch = PromptChatService.buildCurrentChatBranch($ctrl.messages, lastMessageId);
+            function buildBranch(lastMessageId) {
+                return PromptChatService.buildCurrentChatBranch($ctrl.messages, lastMessageId, $ctrl.settingsComparator ? (newMessage, oldMessage) => $ctrl.settingsComparator({ newMessage, oldMessage }) : undefined);
+            }
+
+            function prepareMessages(messages, lastMessageId, customAssistantMessageSettings) {
+                if (customAssistantMessageSettings) {
+                    $ctrl.messages[lastMessageId] = { ...($ctrl.messages[lastMessageId] || {}), $customSettings: customAssistantMessageSettings };
+                }
+
+                $ctrl.messages = PromptChatService.enrichChatMessages(messages, $ctrl.messages);
+                $ctrl.messageBranch = buildBranch(lastMessageId);
                 document.getElementById(entryElId).focus();
                 $timeout(() => {
                     const containerEl = document.getElementById('prompt-chat');
