@@ -219,6 +219,10 @@ else
 	fi
 fi
 
+if [ -n "$stories" ]; then
+	enable_stories="true"
+fi
+
 if [ -z "$stories" -a ! -f /etc/dataiku-cloud -a -d "$DIP_HOME"/stories ]; then
 	echo "[+] $DIP_HOME/stories already exists, will update Stories" >&2
 	stories="true"
@@ -477,6 +481,13 @@ function stories_load {
 		exit 1
 	fi
 
+  if [ -n "$enable_stories" -a "$enable_stories" == "true" ]; then
+    echo "[+] Enable Stories in general settings"
+    DKU_GENERAL_SETTINGS_FILEPATH="$DIP_HOME"/config/general-settings.json
+    sed -E '/"dataikuStoriesSettings"[[:space:]]*:/,/\}/ s/"enabled":[[:space:]]*false/"enabled": true/' "$DKU_GENERAL_SETTINGS_FILEPATH" > "$DKU_GENERAL_SETTINGS_FILEPATH.next"
+    mv "$DKU_GENERAL_SETTINGS_FILEPATH.next" "$DKU_GENERAL_SETTINGS_FILEPATH"
+  fi
+
 	# If image_tag has a value, it is because a version of Stories was specified with the -w flag.
 	# Otherwise, the version of Stories will be the same as the DSS version.
 	if [ "$stories" != "true" ]; then
