@@ -5,7 +5,6 @@ This will retrieve the definition of the stuff to be executed and execute it,
 forking the process (in the image's code environment if necessary).
 """
 
-import dataiku
 from dataiku.base import remoterun
 from dataiku.container.file_exchange import FileExchanger
 from dataiku.core import debugging, intercom
@@ -16,6 +15,8 @@ from dataiku.base.remoterun import safe_os_environ_update
 import requests
 import shutil
 import tempfile
+import socket
+import textwrap
 from dataiku.container.code_studio_server_thread import CommandServerThread
 
 from dataiku.container.compute_resource_usage import ProcessResourceUsageMonitor
@@ -883,15 +884,25 @@ def main():
     elif execution['type'] == 'RECIPE_PREDICTION_STANDALONE_EVAL_PYTHON':
         logging.info("Evaluating external model")
 
+        setup_python_env(execution_id, dku_vars)
+
         logging.info("Running standalone evaluation recipe main")
         run_subprocess(execution_id, '%s -m dataiku.container.exec_standalone_eval_recipe' % python_bin)
 
     elif execution['type'] == 'RECIPE_LLM_EVALUATION_PYTHON':
         logging.info("Evaluating llm model")
 
+        setup_python_env(execution_id, dku_vars)
+
         logging.info("Running llm evaluation recipe main")
         run_subprocess(execution_id, '%s -m dataiku.container.exec_llm_eval_recipe' % python_bin)
 
+    elif execution['type'] == 'RECIPE_AGENT_EVALUATION_PYTHON':
+        logging.info("Evaluating agent")
+
+        logging.info("Running agent evaluation recipe main")
+        run_subprocess(execution_id, '%s -m dataiku.container.exec_agent_eval_recipe' % python_bin)
+
     elif execution['type'] == 'RECIPE_CLUSTERING_CLUSTER_PYTHON':
         logging.info("Clustering dataset")
 
@@ -1170,6 +1181,60 @@ main()
             # no 'failure' per se can happen
             os._exit(0)
 
+    elif execution['type'] == 'WEBAPP_STREAMLIT':
+
+        plugin_id = dku_vars.get("DKU_PLUGIN_ID", None)
+        if plugin_id is not None:
+            logging.info("Getting plugin %s" % plugin_id)
+            fetch_plugin(execution_id, plugin_id)
+
+        definition = json.loads(execution['definition'])
+        project_key = definition['projectKey']
+        webapp_id = definition['webAppId']
+        logging.info("Executing streamlit webapp backend {project_key}/{webapp_id}".format(project_key=project_key, webapp_id=webapp_id))
+
+        if not os.path.exists('backend'):
+            os.mkdir('backend')
+        with open(os.path.join('backend', 'main.py'), 'wb') as fd:
+            fd.write(definition['code'].encode('utf8'))
+        if definition.get('config', None) is not None:
+            if not os.path.exists('.streamlit'):
+                os.mkdir('.streamlit')
+            with open(os.path.join('.streamlit', 'config.toml'), 'wb') as fd:
+                fd.write(definition['config'].encode('utf8'))
+
+        security_info = definition.get('securityInfo', {})
+        web_app_host, web_app_port = setup_webapp_exposition(
+            security_info,
+            project_key,
+            webapp_id,
+            'STREAMLIT',
+            '/__ping'
+        )
+
+        # construct the external url for use in redirects if a redirect is required to capture authentication info
+        nb_workers = definition.get("nbProcesses", 1)
+        start_code = """from dataiku.webapps.run_streamlit import main
+import sys
+sys.argv = [sys.argv[0], './backend/main.py', '-w', '{nb_workers}', '-H', '{web_app_host}', '-p', '{web_app_port}']
+main()
+""".format(nb_workers=nb_workers, web_app_host=web_app_host, web_app_port=web_app_port)
+        with open('start_webapp.py', 'wb') as fd:
+            fd.write(start_code.encode('utf8'))
+
+        with open('/tmp/liveliness.marker', 'w') as f:  # 'w' OK, we write a string
+            f.write('alive')
+
+        setup_python_env(execution_id, dku_vars, False)
+
+        run_metrics_server()
+        logging.info("Executing streamlit webapp {project_key}/{webapp_id} subprocess".format(project_key=project_key, webapp_id=webapp_id))
+        try:
+            run_subprocess(execution_id, '%s -m dataiku.container.exec_py_webapp' % python_bin)
+        finally:
+            # no 'failure' per se can happen
+            os._exit(0)
+
     elif execution['type'] == 'RECIPE_CPYTHON':
         logging.info("Executing Continuous Python recipe")
 
