@@ -11,10 +11,10 @@ from dataiku.doctor import step_constants
 from dataiku.doctor.diagnostics import diagnostics
 from dataiku.doctor.diagnostics.diagnostics import DiagnosticType
 from dataiku.doctor.exception import TimeseriesResamplingException
-from dataiku.doctor.timeseries.preparation.preprocessing import TimeseriesPreprocessing, get_windows_list
+from dataiku.doctor.timeseries.preparation.preprocessing import TimeseriesPreprocessing, get_windows_list, \
+    add_rolling_windows_for_scoring
 from dataiku.doctor.timeseries.models import TimeseriesForecastingAlgorithm
 from dataiku.doctor.timeseries.preparation.resampling.utils import get_frequency
-from dataiku.doctor.timeseries.preparation.preprocessing import add_rolling_windows
 from dataiku.doctor.timeseries.preparation.preprocessing import get_external_features
 from dataiku.doctor.timeseries.preparation.preprocessing import get_filtered_features
 from dataiku.doctor.timeseries.preparation.preprocessing import resample_timeseries
@@ -68,7 +68,8 @@ class TimeseriesScoringHandler(object):
         context = DiagOnlyContext(diagnostics_folder_context) if diagnostics_folder_context else None
         self.listener = ProgressListener(context=context)
 
-        self.timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, self.listener)
+        self.preprocessing_folder_context = preprocessing_folder_context
+        self.timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, self.listener, self.algorithm.SUPPORTS_EXTERNAL_FEATURES)
         if self.external_features:
             self.timeseries_preprocessing.load_resources()
 
@@ -161,45 +162,42 @@ class TimeseriesScoringHandler(object):
 
             # Windowing is performed once and for all on the full dataframe
             windows_list = get_windows_list(self.preprocessing_params) if self.modeling_params.get("isShiftWindowsCompatible", False) else []
-            resampled_df = add_rolling_windows(resampled_df, self.core_params, windows_list)
+            resampled_df = add_rolling_windows_for_scoring(resampled_df, self.core_params, windows_list, self.preprocessing_params, self.preprocessing_folder_context)
 
-        has_future_features = True
+        has_external_features = self.external_features is not None and len(self.external_features) > 0
+        has_future_features = has_external_features and len(get_filtered_features(self.preprocessing_params, include_roles=["INPUT"])) > 0
         if self.modeling_params.get("isShiftWindowsCompatible", False):
-            count_future_features = len(get_filtered_features(self.preprocessing_params, include_roles=["INPUT"]))
-            if count_future_features == 0:
-                has_future_features = False
+            # ML-based models
+            # TODO: extend to algorithms supporting past-only features (role=INPUT_PAST_ONLY)
+            has_preprocessed_features = has_external_features and len(get_filtered_features(self.preprocessing_params, include_roles=["INPUT", "INPUT_PAST_ONLY"])) > 0
+        else:
+            has_preprocessed_features = has_future_features
+        # split past and future df and check that there are enough future external features values if actually required
+        past_resampled_df, future_resampled_df = self._split_past_and_future_df(resampled_df, check_enough_future_values=has_future_features)
 
-        if self.external_features and has_future_features:
-            # split past and future df and check that there are enough future external features values
-
-            past_resampled_df, future_resampled_df = self._split_past_and_future_df(resampled_df, check_enough_future_values=True)
-
-            preprocess_on_full_df = self.algorithm.USE_GLUON_TS
+        preprocess_on_full_df = self.algorithm.USE_GLUON_TS
 
+        if has_preprocessed_features:
             # we could either pass past_resampled_df or future_resampled_df because it will only look at which timeseries identifiers are in the df
             # to retrieve the right resources
             self.timeseries_preprocessing.create_timeseries_preprocessing_handlers(
                 past_resampled_df, preprocess_on_full_df, use_saved_resources=True
             )
-
             past_preprocessed_df = self.timeseries_preprocessing.process(
                 past_resampled_df,
                 step_constants.ProcessingStep.STEP_PREPROCESS_TEST,
                 preprocess_on_full_df,
             )
+        else:
+            past_preprocessed_df = past_resampled_df
 
+        if has_future_features:
             future_preprocessed_df = self.timeseries_preprocessing.process(
                 future_resampled_df,
                 step_constants.ProcessingStep.STEP_PREPROCESS_TEST,
                 preprocess_on_full_df,
             )
-
         else:
-            # even without external features, there can be future values of extra columns that need be added to the output dataset
-            past_resampled_df, future_resampled_df = self._split_past_and_future_df(resampled_df, check_enough_future_values=False)
-
-            # when there are no external features, there are no preprocessing, only past resampling is used for prediction
-            past_preprocessed_df = past_resampled_df
             future_preprocessed_df = None
 
         unseen_timeseries_identifiers = []
