@@ -24,7 +24,7 @@ class SystemPromptSupport(Enum):
 
 # These templates are possibly incorrect , but it doesn't matter much since only old models don't come with a chat template. This is pure legacy code.
 FALCON_JINJA_TEMPLATE = "{% for message in messages %}{% if message.role == 'system' %}>>CONTEXT<<{{ message.content }}\n{% elif message.role == 'user' %}>>QUESTION<<{{ message.content }}\n{% else %}>>ANSWER<<{{ message.content }}\n{% endif %}{% endfor %}"
-MPT_DOLLY_JINJA_TEMPLATE = "{% for message in messages %}{% if message.role == 'system' %}{{ message.content }}\n{% elif message.role == 'user' %}### Instruction:\n{{ message.content }}\n{% else %}### Response:\n{{ message.content }}\n{% endif %}{% endfor %}"
+MPT_JINJA_TEMPLATE = "{% for message in messages %}{% if message.role == 'system' %}{{ message.content }}\n{% elif message.role == 'user' %}### Instruction:\n{{ message.content }}\n{% else %}### Response:\n{{ message.content }}\n{% endif %}{% endfor %}"
 
 
 class ChatTemplateRenderer(object):
@@ -144,15 +144,12 @@ class ChatTemplateRenderer(object):
         """
 
         # Fallbacks
-        if self.hf_handling_mode == "TEXT_GENERATION_DOLLY":
-            logger.info("Using Dolly template")
-            return MPT_DOLLY_JINJA_TEMPLATE
         if self.hf_handling_mode == "TEXT_GENERATION_FALCON":
             logger.info("Using Falcon template")
             return FALCON_JINJA_TEMPLATE
         if self.hf_handling_mode == "TEXT_GENERATION_MPT":
             logger.info("Using MPT template")
-            return MPT_DOLLY_JINJA_TEMPLATE
+            return MPT_JINJA_TEMPLATE
 
         return self._get_chat_template(tools)
 
@@ -368,14 +365,12 @@ class ChatTemplateRenderer(object):
                 new_messages.append(current_message)
                 current_message = deepcopy(message)
             else:
-                if "content" in message and message["content"] is not None:
-                    if "content" in current_message and current_message["content"]:
-                        current_message["content"] += "\n"
-                    else:
-                        current_message["content"] = ""
-                    current_message["content"] += message["content"]
-                if "parts" in message and message["parts"] is not None:
-                    current_message["parts"] = current_message.get("parts", []) or []
+                if "content" in current_message:
+                    text = current_message.pop("content")
+                    current_message["parts"] = [{"type": "TEXT", "text": text}] if text else []
+                if message.get("content"):
+                    current_message["parts"].extend([{"type": "TEXT", "text": message["content"] }])
+                if message.get("parts"):
                     current_message["parts"].extend(message["parts"])
 
         if current_message is not None:
