@@ -17,6 +17,8 @@ from dataiku.huggingface.types import ProcessSinglePromptResponseTextFull
 from dataiku.huggingface.types import ProcessSinglePromptResponseZeroShotClassification
 from dataiku.huggingface.types import ProcessSingleEmbeddingCommand
 from dataiku.huggingface.types import ProcessSingleEmbeddingResponse
+from dataiku.huggingface.types import ProcessSingleRerankingCommand
+from dataiku.huggingface.types import ProcessSingleRerankingResponse
 from dataiku.huggingface.types import SingleCommand
 from dataiku.huggingface.types import SingleResponse
 
@@ -60,6 +62,7 @@ class ModelPipeline(ABC, Generic[SingleCommand, SingleResponse]):
         # Pipeline can optionally produce tracking data (collected via WT1)
         # Do NOT store any sensitive information into this field, not even model names.
         self.model_tracking_data: ModelTrackingData = {}
+        self.used_engine = ""
 
     async def initialize_model(self):
         """Optionally perform additionnal setup and checks after loading the model in memory"""
@@ -74,11 +77,12 @@ class ModelPipeline(ABC, Generic[SingleCommand, SingleResponse]):
 class MockModelPipelineTextGenerationOrSummarization(ModelPipeline[ProcessSinglePromptCommand, ProcessSinglePromptResponseText]):
     """Mock pipeline for text generation or text summarization."""
 
-    def __init__(self):
+    def __init__(self, model_settings):
         logger.info("Instantiating mock pipeline for text generation or summarization")
         super().__init__()
         self.request_counter = 0
-        self.model_tracking_data["used_engine"] = "mock"
+        self.used_engine = "mock"
+        self.model_settings = model_settings
 
     async def initialize_model(self):
         logger.info("Running test prompt for mocked text generation or summarization pipeline")
@@ -91,6 +95,11 @@ class MockModelPipelineTextGenerationOrSummarization(ModelPipeline[ProcessSingle
 
         await asyncio.sleep(random.uniform(0, 2))
         prompt_start = request["prompt"][:20]
+        if request.get("stream") and self.model_settings.get("reasoning_parser") is not None:
+            yield self.generate_reasoning_dss_chunk("I'm deeply thinking")
+            await asyncio.sleep(random.uniform(0, 2))
+            yield self.generate_reasoning_dss_chunk("but my brain is empty")
+            await asyncio.sleep(random.uniform(0, 2))
         if request.get("stream"):
             yield { "chunk": { "text": "hello! " } }
             await asyncio.sleep(random.uniform(0, 1))
@@ -109,6 +118,20 @@ class MockModelPipelineTextGenerationOrSummarization(ModelPipeline[ProcessSingle
             yield {"text": f"salut c'est moi de l'autre cote (request #{request_index}) [{prompt_start}]",
                    "usage": usage}
 
+    def generate_reasoning_dss_chunk(self, text):
+        return { "chunk":
+            { "artifacts": [{
+                "type": "REASONING",
+                "index": 0,
+                "parts": [
+                    {
+                        "index": 0,
+                        "text": text,
+                        "type": "TEXT"
+                    }]
+            }]
+            }
+        }
 
 class MockModelPipelineTextClassification(ModelPipeline[ProcessSinglePromptCommand, ProcessSinglePromptResponseTextFull]):
     """Mock pipeline for text model-provided classification."""
@@ -116,7 +139,7 @@ class MockModelPipelineTextClassification(ModelPipeline[ProcessSinglePromptComma
     def __init__(self):
         logger.info("Instantiating mock pipeline for text classification")
         super().__init__()
-        self.model_tracking_data["used_engine"] = "mock"
+        self.used_engine = "mock"
 
     async def run_single_async(self, request: ProcessSinglePromptCommand) -> AsyncIterator[ProcessSinglePromptResponseTextFull]:
         await asyncio.sleep(random.uniform(0, 2))
@@ -137,7 +160,7 @@ class MockModelPipelineZeroShotClassification(ModelPipeline[ProcessSinglePromptC
     def __init__(self):
         logger.info("Instantiating mock pipeline for zero shot classification")
         super().__init__()
-        self.model_tracking_data["used_engine"] = "mock"
+        self.used_engine = "mock"
 
     async def run_single_async(self, request: ProcessSinglePromptCommand) -> AsyncIterator[ProcessSinglePromptResponseZeroShotClassification]:
         await asyncio.sleep(random.uniform(0, 2))
@@ -159,7 +182,7 @@ class MockModelPipelineEmbedding(ModelPipeline[ProcessSingleEmbeddingCommand, Pr
     def __init__(self):
         logger.info("Instantiating mock pipeline for embedding")
         super().__init__()
-        self.model_tracking_data["used_engine"] = "mock"
+        self.used_engine = "mock"
 
     async def initialize_model(self):
         logger.info("Running test prompt for mocked embedding pipeline")
@@ -171,16 +194,38 @@ class MockModelPipelineEmbedding(ModelPipeline[ProcessSingleEmbeddingCommand, Pr
         embedding_size = 10
         yield {"embedding": [random.random() for _ in range(embedding_size)]}
 
+class MockModelPipelineReranking(ModelPipeline[ProcessSingleRerankingCommand, ProcessSingleRerankingResponse]):
+    """Mock pipeline for reranking."""
 
-def create_mock_pipeline(hf_handling_mode):
+    def __init__(self):
+        logger.info("Instantiating mock pipeline for reranking")
+        super().__init__()
+        self.model_tracking_data["used_engine"] = "mock"
+
+    async def initialize_model(self):
+        logger.info("Running test prompt for mocked reranking pipeline")
+        await asyncio.sleep(random.uniform(0, 2))
+        logger.info("Test prompt for mocked reranking pipeline ran successfully")
+
+    async def run_single_async(self, request: ProcessSingleRerankingCommand) -> AsyncIterator[ProcessSingleRerankingResponse]:
+        await asyncio.sleep(random.uniform(0, 2))
+        unordered_reranked_documents = [{"index": idx, "relevanceScore": len(doc.get("content", ""))} for idx, doc in enumerate(request["documents"])]
+        reranked_documents = sorted(unordered_reranked_documents, key=lambda x: x["relevanceScore"])
+        yield {"documents": reranked_documents}
+
+
+
+def create_mock_pipeline(hf_handling_mode, model_settings):
     if hf_handling_mode == "ZSC_GENERIC":
         return MockModelPipelineZeroShotClassification()
-    elif hf_handling_mode in ["TEXT_CLASSIFICATION_SENTIMENT", "TEXT_CLASSIFICATION_EMOTIONS", "TEXT_CLASSIFICATION_TOXICITY", "TEXT_CLASSIFICATION_PROMPT_INJECTION", "TEXT_CLASSIFICATION_OTHER"]:
+    elif hf_handling_mode.startswith("TEXT_CLASSIFICATION"):
         return MockModelPipelineTextClassification()
-    elif hf_handling_mode in ["SUMMARIZATION_GENERIC", "SUMMARIZATION_ROBERTA", "T5"] or hf_handling_mode.startswith("TEXT_GENERATION_"):
-        return MockModelPipelineTextGenerationOrSummarization()
+    elif hf_handling_mode == "SUMMARIZATION_GENERIC" or hf_handling_mode.startswith("TEXT_GENERATION_"):
+        return MockModelPipelineTextGenerationOrSummarization(model_settings)
     elif hf_handling_mode in ["TEXT_EMBEDDING", "IMAGE_EMBEDDING"]:
         return MockModelPipelineEmbedding()
+    elif hf_handling_mode in ["DOCUMENT_RERANKING"]:
+        return MockModelPipelineReranking()
     else:
         raise Exception("Unknown handling mode for mock pipeline (hf_handling_mode={hf_handling_mode})".format(hf_handling_mode=hf_handling_mode))
 
@@ -223,9 +268,31 @@ def create_model_pipeline(
                 model_kwargs = {"use_safetensors": False}
                 break
 
-    requested_engine = "AUTO"
-    requested_quantization = "NONE"
-    # Set quantization params and trust_remote_code if needed and handle text engine
+    # RESOLVE ENGINE
+    requested_engine = model_settings.get("engine", "AUTO")
+    engine = requested_engine
+
+    #   RESOLVE ENGINE FOR TEXT GENERATION AND TEXT EMBEDDING
+    if hf_handling_mode.startswith("TEXT_GENERATION_") or hf_handling_mode == "TEXT_EMBEDDING":
+        from dataiku.huggingface.vllm_backend.utils import is_vllm_supported
+        can_use_vllm = is_vllm_supported(hf_handling_mode, model_name_or_path, base_model_name_or_path, model_settings, expected_vllm_version)
+
+        logger.info(f"Requested engine: {engine}")
+        if engine == "AUTO":
+            engine = "VLLM" if can_use_vllm else "TRANSFORMERS"
+        logger.info(f"Selected engine: {engine}")
+
+    #   RESOLVE ENGINE FOR DOCUMENT RERANKING
+    if hf_handling_mode == "DOCUMENT_RERANKING":
+        logger.info(f"Requested engine: {engine}")
+        if engine == "VLLM":
+            logger.info(f"Requested engine 'VLLM' not supported for {hf_handling_mode}. Using 'AUTO' instead")
+            engine = "AUTO"
+        if engine == "AUTO":
+            engine = "TRANSFORMERS"
+            logger.info(f"Selected 'TRANSFORMERS' engine for reranking.")
+
+    # TEXT GENERATION
     if hf_handling_mode.startswith("TEXT_GENERATION_"):
         from dataiku.huggingface.pipeline_text_gen import ModelPipelineTextGeneration
         requested_quantization = model_settings["quantizationMode"]
@@ -233,76 +300,78 @@ def create_model_pipeline(
         if model_settings.get("trustRemoteCode"):
             model_kwargs['trust_remote_code'] = True
 
-        from dataiku.huggingface.pipeline_text_gen_vllm import ModelPipelineTextGenerationVLLM
-        can_use_vllm = ModelPipelineTextGenerationVLLM.supports_model(hf_handling_mode, model_name_or_path, base_model_name_or_path, model_settings, expected_vllm_version)
-        requested_engine = model_settings['engine']
-        engine = requested_engine
-        logger.info(f"Requested engine: {engine}")
-        if engine == "AUTO":
-            engine = "VLLM" if can_use_vllm else "TRANSFORMERS"
-        logger.info(f"Selected engine: {engine}")
         if engine == "VLLM":
+            from dataiku.huggingface.pipeline_text_gen_vllm import ModelPipelineTextGenerationVLLM
             pipeline = ModelPipelineTextGenerationVLLM(hf_handling_mode, model_name_or_path, base_model_name_or_path, model_settings, use_dss_model_cache, supports_image_inputs)
             pipeline.model_tracking_data["requested_engine"] = requested_engine
             pipeline.model_tracking_data["requested_quantization"] = requested_quantization
             return pipeline
 
-    if model_settings['engine'] == "VLLM":
-        logger.info(f"Requested engine 'VLLM' not supported for {hf_handling_mode}. Using 'TRANSFORMERS' instead")
+        if base_model_name_or_path is not None:
+            return ModelPipelineTextGenerationLora(model_name_or_path, model_kwargs, batch_size, hf_handling_mode, base_model_name_or_path)
 
-    from dataiku.huggingface.pipeline_text_gen_lora import ModelPipelineTextGenerationLora
-    if base_model_name_or_path is not None:
-        return ModelPipelineTextGenerationLora(model_name_or_path, model_kwargs, batch_size, hf_handling_mode, base_model_name_or_path)
-
-    if hf_handling_mode == "ZSC_GENERIC":
-        from dataiku.huggingface.pipeline_zero_shot_classif import ModelPipelineZeroShotClassif
-        return ModelPipelineZeroShotClassif(model_name_or_path, model_kwargs, batch_size)
-    elif hf_handling_mode in ["TEXT_CLASSIFICATION_SENTIMENT", "TEXT_CLASSIFICATION_EMOTIONS", "TEXT_CLASSIFICATION_TOXICITY", "TEXT_CLASSIFICATION_PROMPT_INJECTION", "TEXT_CLASSIFICATION_OTHER"]:
-        from dataiku.huggingface.pipeline_text_classif import ModelPipelineTextClassif
-        return ModelPipelineTextClassif(model_name_or_path, model_kwargs, batch_size)
-
-    elif hf_handling_mode == "SUMMARIZATION_GENERIC":
-        from dataiku.huggingface.pipeline_summarization_generic import ModelPipelineSummarizationGeneric
-        return ModelPipelineSummarizationGeneric(model_name_or_path, model_kwargs, batch_size)
-    elif hf_handling_mode == "SUMMARIZATION_ROBERTA":
-        from dataiku.huggingface.pipeline_summarization_roberta import ModelPipelineSummarizationRoberta
-        return ModelPipelineSummarizationRoberta(model_name_or_path, model_kwargs, batch_size)
-    elif hf_handling_mode in ["TEXT_GENERATION_DEEPSEEK", "TEXT_GENERATION_FALCON", "TEXT_GENERATION_LLAMA_2", "TEXT_GENERATION_LLAMA_GUARD", "TEXT_GENERATION_MISTRAL", "TEXT_GENERATION_QWEN", "TEXT_GENERATION_ZEPHYR", "TEXT_GENERATION_DOLLY", "TEXT_GENERATION_GEMMA", "TEXT_GENERATION_PHI_3", "TEXT_GENERATION_AUTO", "TEXT_GENERATION_GPT"]:
-        from dataiku.huggingface.pipeline_text_gen import ModelPipelineTextGeneration
-        if hf_handling_mode == "TEXT_GENERATION_DOLLY":
-            logging.warning("Batching is not yet supported for Dolly, handling each prompt separately.")
-            batch_size = 1
         # This is needed until a new version of transformers fixes the following issue,
         # see : https://huggingface.co/microsoft/Phi-3-mini-4k-instruct/discussions/82
         if hf_handling_mode == "TEXT_GENERATION_PHI_3":
             model_kwargs['trust_remote_code'] = True
+
+        if hf_handling_mode == "TEXT_GENERATION_MPT":
+            from dataiku.huggingface.pipeline_text_gen_mpt import ModelPipelineTextGenerationMPT
+            pipeline = ModelPipelineTextGenerationMPT(model_name_or_path, model_kwargs, batch_size, model_settings, not untrusted_model)
+            pipeline.model_tracking_data["requested_engine"] = requested_engine
+            pipeline.model_tracking_data["requested_quantization"] = requested_quantization
+            return pipeline
+
         pipeline = ModelPipelineTextGeneration(model_name_or_path, model_kwargs, batch_size, hf_handling_mode)
         pipeline.model_tracking_data["requested_engine"] = requested_engine
         pipeline.model_tracking_data["requested_quantization"] = requested_quantization
         return pipeline
-    elif hf_handling_mode == "TEXT_GENERATION_MPT":
-        from dataiku.huggingface.pipeline_text_gen_mpt import ModelPipelineTextGenerationMPT
-        pipeline = ModelPipelineTextGenerationMPT(model_name_or_path, model_kwargs, batch_size, model_settings, not untrusted_model)
+
+    trust_remote_code = model_settings.get("trustRemoteCode") or False
+
+    # TEXT EMBEDDING
+    if hf_handling_mode == "TEXT_EMBEDDING":
+        if engine == "VLLM":
+            from dataiku.huggingface.pipeline_text_embed_vllm import ModelPipelineTextEmbeddingVLLM
+            pipeline = ModelPipelineTextEmbeddingVLLM(model_name_or_path, model_settings)
+        else:
+            from dataiku.huggingface.pipeline_text_embed_extraction import ModelPipelineTextEmbeddingExtraction
+            pipeline = ModelPipelineTextEmbeddingExtraction(model_name_or_path, trust_remote_code, batch_size)
+
         pipeline.model_tracking_data["requested_engine"] = requested_engine
-        pipeline.model_tracking_data["requested_quantization"] = requested_quantization
         return pipeline
-    elif hf_handling_mode == "TEXT_GENERATION_GENERIC":
-        from dataiku.huggingface.pipeline_text_gen_generic import ModelPipelineTextGenerationGeneric
-        pipeline = ModelPipelineTextGenerationGeneric(model_name_or_path, model_kwargs, batch_size)
+
+    # DOCUMENT RERANKING
+    if hf_handling_mode == "DOCUMENT_RERANKING":
+        if engine == "TRANSFORMERS":
+            from dataiku.huggingface.pipeline_reranking_transformers import ModelPipelineRerankingTransformers
+            pipeline = ModelPipelineRerankingTransformers(model_name_or_path, trust_remote_code, batch_size)
+        else:
+            # TODO @reranking support VLLM
+            # Should be unreachable due to engine resolution above unless the user explicitly sets a random value
+            raise Exception(f"Requested engine '{engine}' not supported for {hf_handling_mode}")
+
         pipeline.model_tracking_data["requested_engine"] = requested_engine
-        pipeline.model_tracking_data["requested_quantization"] = requested_quantization
+        pipeline.model_tracking_data["engine"] = engine
         return pipeline
-    elif hf_handling_mode == "T5":
-        from dataiku.huggingface.pipeline_text_gen_t5 import ModelPipelineTextGenerationT5
-        return ModelPipelineTextGenerationT5(model_name_or_path, model_kwargs, batch_size)
-    elif hf_handling_mode == "TEXT_EMBEDDING":
-        from dataiku.huggingface.pipeline_text_embed_extraction import ModelPipelineTextEmbeddingExtraction
-        trust_remote_code = False if model_settings.get("trustRemoteCode") is None else model_settings.get("trustRemoteCode")
-        return ModelPipelineTextEmbeddingExtraction(model_name_or_path, trust_remote_code, batch_size)
-    elif hf_handling_mode == "IMAGE_EMBEDDING":
+
+    # OTHER PIPELINES
+    if requested_engine == "VLLM":
+        logger.info(f"Requested engine 'VLLM' not supported for {hf_handling_mode}. Using 'TRANSFORMERS' instead")
+
+    if hf_handling_mode == "ZSC_GENERIC":
+        from dataiku.huggingface.pipeline_zero_shot_classif import ModelPipelineZeroShotClassif
+        return ModelPipelineZeroShotClassif(model_name_or_path, model_kwargs, batch_size)
+    if hf_handling_mode.startswith("TEXT_CLASSIFICATION"):
+        from dataiku.huggingface.pipeline_text_classif import ModelPipelineTextClassif
+        return ModelPipelineTextClassif(model_name_or_path, model_kwargs, batch_size)
+    if hf_handling_mode == "SUMMARIZATION_GENERIC":
+        from dataiku.huggingface.pipeline_summarization_generic import ModelPipelineSummarizationGeneric
+        return ModelPipelineSummarizationGeneric(model_name_or_path, model_kwargs, batch_size)
+    if hf_handling_mode == "IMAGE_EMBEDDING":
         from dataiku.huggingface.pipeline_image_embed_extraction import ModelPipelineImageEmbeddingExtraction
         return ModelPipelineImageEmbeddingExtraction(model_name_or_path, hf_model_name, use_dss_model_cache, batch_size)
-    elif hf_handling_mode == "IMAGE_GENERATION_DIFFUSION":
+    if hf_handling_mode == "IMAGE_GENERATION_DIFFUSION":
         from dataiku.huggingface.pipeline_image_gen_diffusion import ModelPipelineImageGenerationDiffusion
         return ModelPipelineImageGenerationDiffusion.create_pipeline(model_name_or_path, model_settings, use_dss_model_cache, batch_size)
 
