@@ -96,6 +96,16 @@ class DSSLLM(object):
     def new_images_generation(self):
         return DSSLLMImageGenerationQuery(self)
 
+
+    def new_reranking(self):
+        """
+        Create a new reranking query.
+
+        :returns: A handle on the generated reranking query.
+        :rtype: :class:`DSSLLMRerankingQuery`
+        """
+        return DSSLLMRerankingQuery(self)
+
     def as_langchain_llm(self, **data):
         """
         Create a langchain-compatible LLM object for this LLM.
@@ -1094,3 +1104,118 @@ class DSSLLMImageGenerationResponse(object):
         :rtype: List[bytes]
         """
         return self.get_images(as_type="bytes")
+
+class DSSLLMRerankingQuery(object):
+    """
+    A handle to interact with a reranking query.
+    Reranking queries allow you to send a text query and a list of documents to a DSS-managed ranking model
+    and retrieve the documents ranked according to their relevance to the query.
+
+    .. important::
+        Do not create this class directly, use :meth:`dataikuapi.dss.llm.DSSLLM.new_reranking` instead.
+    """
+    def __init__(self, llm):
+        self.llm = llm
+        self.rq = {
+            "queryParts": [],
+            "documents": [],
+        }
+
+    def with_query(self, text):
+        """
+        Sets the reranking text query.
+
+        :param str text: The reranking text query.
+        """
+        self.rq["queryParts"].append({ "text": text, "type": "TEXT" })
+        return self
+
+    def with_document(self, text):
+        """
+        Adds a text document to the list of documents to be reranked.
+
+        :param str text: The text document to be reranked.
+        """
+        self.rq["documents"].append({"parts": [ { "text": text, "type": "TEXT" } ] })
+        return self
+
+    def execute(self):
+        """
+        Run the reranking query and retrieve the LLM response.
+
+        :returns: The LLM response.
+        :rtype: :class:`DSSLLMRerankingResponse`
+        """
+        reranking_query = {
+            "llmId": self.llm.llm_id,
+            "queries": [self.rq]
+        }
+        ret = self.llm.client._perform_json("POST", "/projects/%s/llms/rerankings" % (self.llm.project_key), body=reranking_query)
+        return DSSLLMRerankingResponse(ret)
+
+class DSSLLMRerankingResponse(object):
+    """
+    A handle to interact with a ranking query result.
+
+    .. important::
+        Do not create this class directly, use :meth:`dataikuapi.dss.llm.DSSLLMRerankingQuery.execute` instead.
+    """
+    def __init__(self, raw_resp):
+        self._raw = raw_resp
+        self._resp = self._raw["responses"][0]
+
+    @property
+    def success(self):
+        """
+        :return: The outcome of the reranking query.
+        :rtype: bool
+        """
+        return self._resp["ok"]
+
+    @property
+    def error_message(self):
+        """
+        :return: The error message if the reranking query failed, None otherwise.
+        :rtype: Union[str, None]
+        """
+        if not self.success:
+            return self._resp.get("errorMessage", "Unknown error")
+        return None
+
+    @property
+    def documents(self):
+        """
+        The array of reranked documents.
+        :rtype: list[:class:`DSSLLMRerankingResponse.RankedDocument`]
+        """
+        if not self.success:
+            raise Exception("Reranking request failed: %s" % self.error_message)
+        return [self.RankedDocument(raw_doc=ranked_doc) for ranked_doc in self._resp["documents"]]
+
+    @property
+    def trace(self):
+        """
+        :return: The trace of the reranking query if available, None otherwise.
+        :rtype: Union[dict, None]
+        """
+        return self._resp.get("trace")
+
+    class RankedDocument(object):
+        def __init__(self, raw_doc):
+            self._raw_doc = raw_doc
+
+        @property
+        def index(self):
+            """
+            :return: The index of the document in the original request.
+            :rtype: int
+            """
+            return self._raw_doc["index"]
+
+        @property
+        def relevance_score(self):
+            """
+            :return: The relevance score assigned to the document by the ranking model.
+            :rtype: float
+            """
+            return self._raw_doc["relevanceScore"]
