@@ -20,18 +20,18 @@ def build_config() -> dict[str, str]:
 
         # Package versions
         contourpy_version="1.3.3",
-        meson_version="1.9.1",
+        meson_version="1.10.0",
         mesonpy_version="0.18.0",
         pybind11_version="3.0.1",
 
         # Misc meson settings
         meson_backend="ninja",
-        build_dir=r"/tmp/pip-install-fwikmp1q/contourpy_02be2a4a36664edc87d9962141ecba46/.mesonpy-h3_a17kl/lib/contourpy/util",
-        source_dir=r"/tmp/pip-install-fwikmp1q/contourpy_02be2a4a36664edc87d9962141ecba46/lib/contourpy/util",
+        build_dir=r"/tmp/pip-install-40er9dp9/contourpy_fc37cd599cb0413aadb65107c1c0119f/.mesonpy-r_8_n75t/lib/contourpy/util",
+        source_dir=r"/tmp/pip-install-40er9dp9/contourpy_fc37cd599cb0413aadb65107c1c0119f/lib/contourpy/util",
         cross_build="False",
 
         # Build options
-        build_options=r"-Dbuildtype=release -Db_ndebug=if-release -Db_vscrt=md -Dvsenv=True --native-file=/tmp/pip-install-fwikmp1q/contourpy_02be2a4a36664edc87d9962141ecba46/.mesonpy-h3_a17kl/meson-python-native-file.ini",
+        build_options=r"-Dbuildtype=release -Db_ndebug=if-release -Db_vscrt=md -Dvsenv=True --native-file=/tmp/pip-install-40er9dp9/contourpy_fc37cd599cb0413aadb65107c1c0119f/.mesonpy-r_8_n75t/meson-python-native-file.ini",
         buildtype="release",
         cpp_std="c++17",
         debug="False",
